from oologfcn import OpenOptException
from numpy import ndarray, matrix, asscalar, asarray, asfarray
from time import time, clock, strftime

class OpenOptResult: 
    # TODO: implement it
    #extras = EmptyClass() # used for some optional output
    def __call__(self, *args):
        if not self.isFDmodel:
            raise OpenOptException('Is callable for FuncDesigner models only')
        r = []
        for arg in args:
            tmp = [(self._xf[elem] if isinstance(elem,  str) \
                    else self.xf[elem]) for elem in (arg.tolist() if isinstance(arg, ndarray) \
                    else arg if type(arg) in (tuple, list) else [arg])]
            tmp = [asscalar(item) if type(item) in (ndarray, matrix) and item.size == 1 \
                   #else item[0] if type(item) in (list, tuple) and len(item) == 0 \
                   else item for item in tmp]
            r.append(tmp if type(tmp) not in (list, tuple) or len(tmp)!=1 else tmp[0])
        r = r[0] if len(args) == 1 else r
        if len(args) == 1 and type(r) in (list, tuple) and len(r) >1: 
            r = asfarray(r, dtype = type(r[0]))
        return r
        
    def __init__(self, p):
        self.rf = asscalar(asarray(p.rf))
        self.ff = asscalar(asarray(p.ff))
        self.isFDmodel = p.isFDmodel
        self.probType = p.probType
        self.probName = p.name
        from openopt import oosolver
        self.solverName = oosolver(p.solver).__name__
        
        self._misc = {}
        Arr = ['_localtime_started', '_localtime_finished', 
                     'isObjFunValueASingleNumber', 'isFDmodel', 'err', '_bg_color']
        if p.isFDmodel:
            Arr += ['_init_ooarrays',
                     '_init_fixed_ooarrays', 'freeVarsSet', 'fixedVarsSet',
                     '_init_fixed_ooarrays']
        for attr in Arr:
                        self._misc[attr] = getattr(p, attr)
        
        
#        self.p = p
#        self.name = p.name
        if p.probType == 'EIG':
            self.eigenvalues, self.eigenvectors = p.eigenvalues, p.eigenvectors
        
        if p.isFDmodel:
            from FuncDesigner import oopoint
            self.xf = dict((v, asscalar(val) if isinstance(val, ndarray) and val.size ==1 \
                            else dict((field, v.domain[int(val)][j]) for j, field in enumerate(v.fields)) if v.fields != ()\
                            else v.reverse_aux_domain[int(val)] if 'reverse_aux_domain' in v.__dict__\
                            else v.aux_domain[val] if 'aux_domain' in v.__dict__ else val) \
                            for v, val in p.xf.items())
            if not hasattr(self, '_xf'):
                #self._xf = dict([(v.name, asscalar(val) if isinstance(val, ndarray) and val.size ==1 else val) for v, val in p.xf.items()])
                self._xf = dict((v.name, val) for v, val in self.xf.items())
            self.xf = oopoint(self.xf, maxDistributionSize = p.maxDistributionSize, skipArrayCast = True)
        else:
            self.xf = p.xf
        
        # TODO: mb use the fields in MOP
        if p.probType == 'MOP':
            for attr in ('xf', 'ff', 'rf', '_xf'):
                delattr(self, attr)

        
        #if len(p.solutions) == 0 and p.isFeas(p.xk): p.solutions = [p.xk]
        
        # TODO: mb perform check on each solution for more safety?
        # although it can slow down calculations for huge solutions number
        #self.solutions = p.solutions 

        self.elapsed = dict()
        self.elapsed['solver_time'] = round(100.0*(time() - p.timeStart))/100.0
        self.elapsed['solver_cputime'] = round(100.0*(clock() - p.cpuTimeStart))/100.0
        self.elapsed['initialization_time'] = round(100.0*p.initTime)/100.0
        self.elapsed['initialization_cputime'] = round(100.0*p.initCPUTime)/100.0

        for fn in ('ff', 'istop', 'duals', 'isFeasible', 'msg', 'stopcase', 'iterValues',  'special', 'extras', 'solutions'):
            if hasattr(p, fn):  setattr(self, fn, getattr(p, fn))

        #TODO: mention in doc; mb somehow remove for large sequence of subprobs to clear memory
        if hasattr(p.solver, 'innerState'):
            self.extras['innerState'] = p.solver.innerState

        self.solverInfo = dict()
        for fn in ('homepage',  'alg',  'authors',  'license',  'info', 'name'):
            self.solverInfo[fn] =  getattr(p.solver,  '__' + fn + '__')

            # note - it doesn't work for len(args)>1 for current Python ver  2.6
            #self.__getitem__ = c # = self.__call__
            
        if p.plot:
            #for df in p.graphics.drawFuncs: df(p)    #TODO: include time spent here to (/cpu)timeElapsedForPlotting
            self.elapsed['plot_time'] = round(100*p.timeElapsedForPlotting[-1])/100 # seconds
            self.elapsed['plot_cputime'] = p.cpuTimeElapsedForPlotting[-1]
        else:
            self.elapsed['plot_time'] = 0
            self.elapsed['plot_cputime'] = 0

        self.elapsed['solver_time'] -= self.elapsed['plot_time']
        self.elapsed['solver_cputime'] -= self.elapsed['plot_cputime']

        self.evals = dict((key, val if type(val) == int else round(val *10) /10.0) for key, val in p.nEvals.items())
        self.evals['iter'] = p.iter
        
    _encode = lambda self: None # overloaded in some subclasses

    def _tkview(self, w=None):
        r = self
#        p = r.p
        try:
            from GUI import Text, Scrollbar, Tk, Button
        except:
            self._misc['err']('''
            Tkinter is not installed. 
            If you have Linux you could try using "apt-get install python-tk"
            ''')
        useRoot = w is None
        if useRoot:
            from openopt import __version__ as ooversion
            w = Tk()
            w.wm_title('OpenOpt  ' + ooversion)
            C = Button(w, text = 'Close', command = w.destroy)
            C.pack(side='bottom', fill='both', expand=True)
        if not getattr(w, 'made', False): 
            scrollbar = Scrollbar(w)
            R = Text(w,  yscrollcommand=scrollbar.set, bg=self._misc['_bg_color'])
            R.pack(side = 'left', expand=True, fill='both')
            scrollbar.pack(side='right', fill='y')
            scrollbar.config(command=R.yview)
            
            rr = _txtrepr(r)
            for elem in rr:
                R.insert('end', elem + '\n')
            
            w.made = True
        else:
            w.pack()#is it reachable?
        if useRoot:
            w.mainloop()
    
    def __getattr__(self, attr):
        if attr == 'view':
            print(''.join([elem + '\n' for elem in _txtrepr(self)]))
        elif attr == 'tk':
            self._tkview()
        else:
            raise AttributeError('''
            you are trying to obtain incorrect attribute "%s" 
            from OpenOpt result struct''' % attr
            )
        
def _txtrepr(r):
    rr = []
    if r is None:
        rr.append('result is None, no other info is available')
        return
        
    rr.append('Name: ' +  r.probName)
    rr.append('Type: ' +  r.probType)
    
    M = r._misc
    _localtime_started, _localtime_finished, \
     isObjFunValueASingleNumber, isFDmodel  =     M['_localtime_started'], M['_localtime_finished'], \
     M['isObjFunValueASingleNumber'], M['isFDmodel']
    
    rr.append('Solver: ' +  r.solverName)
#        R.highlight_pattern("word", foreground="red")
    rr.append(strftime("Started: \t%a, %d %b %Y %H:%M:%S", _localtime_started))
    rr.append(strftime("Finished:\t%a, %d %b %Y %H:%M:%S", _localtime_finished))
    
    if isObjFunValueASingleNumber:
        rr.append('Objective value: %0.9g' %  r.ff)
        if 'extremumBounds' in r.extras:
            t = r.extras['extremumBounds']
            rr.append('Estimated |f - f*|: %0.2g' % (t[1]-t[0]))
    
    for fn in ('isFeasible', 'stopcase','istop', 'msg'):
        rr.append(fn +': ' + str(getattr(r, fn)))
    
    rr.append('-'*45)
    # TODO: MCP etc
    if isFDmodel:
        _init_ooarrays,\
        _init_fixed_ooarrays, freeVarsSet, fixedVarsSet,\
        _init_fixed_ooarrays = M['_init_ooarrays'],\
     M['_init_fixed_ooarrays'], M['freeVarsSet'], M['fixedVarsSet'],\
     M['_init_fixed_ooarrays']
    if isFDmodel and r.probType not in ('KSP', 'BPP', 'TSP'):
        from FuncDesigner import _Stochastic

        '''                                           Free vars                                                         '''
        usedVars = set()
        free_ooarrays = _init_ooarrays.difference(_init_fixed_ooarrays)
        for v in free_ooarrays:
            if v.name.startswith('unnamed') and not v[0].name.startswith('unnamed'):
                for _v in v.view(ndarray):
                    if isinstance(_v(r), _Stochastic): continue
                    rr.append(_v.name +': ' + str(_v(r)))
            else:
                if isinstance(v[0](r), _Stochastic): continue #TODO: check other array elements
                rr.append(v.name +': ' + str(v(r).tolist()))
            usedVars.update(v.view(ndarray).tolist())
        for v in freeVarsSet.difference(usedVars):
            V = v(r)
            if isinstance(V, _Stochastic): continue
            rr.append(v.name +': ' + str(V if type(V)!=ndarray else V.tolist()))
        rr.sort(key = lambda elem:len(elem))

        '''                                           Fixed vars                                                         '''
        if len(fixedVarsSet) != 0:
            usedVars = set()
            fixed_ooarrays = _init_fixed_ooarrays
            for v in fixed_ooarrays:
                if v.name.startswith('unnamed') and not v[0].name.startswith('unnamed'):
                    for _v in v.view(ndarray):
                        if isinstance(_v(r), _Stochastic): continue
                        rr.append(_v.name +': ' + str(_v(r)))
                else:
                    if isinstance(v[0](r), _Stochastic): continue#TODO: check other array elements
                    rr.append(v.name +': ' + str(v(r).tolist()))
                usedVars.update(v.view(ndarray).tolist())
            for v in fixedVarsSet.difference(usedVars):
                V = v(r)
                if isinstance(V, _Stochastic): continue
                rr.append(v.name +': ' + str(V if type(V)!=ndarray else V.tolist()))
#            rr.sort(key = lambda elem:len(elem))
            if len(rr):
                rr.append('\n'+'-'*13+'  fixed variables  ' + '-'*13)


    elif r.probType == 'TSP' and hasattr(r, 'Edges'): # MOP
        for elem in [r.nodes, r.edges, r.Edges]:
            rr.append('\n' + str(elem))
            if elem is not r.Edges:
                rr.append('\n'+'-'*45)
    else:
        rr.append(str(r.xf))
    return rr



#from numpy import asarray,  ones, all, isfinite, copy, nan, concatenate, array, dot
#from openopt.kernel.ooMisc import WholeRepr2LinConst, xBounds2Matrix
from openopt.kernel.setDefaultIterFuncs import SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON,  SMALL_DELTA_X, SMALL_DELTA_F
from numpy import isfinite, asscalar, asfarray, abs, copy, isinf, ones, array
import nlopt

def NLOPT_AUX(p, solver, opts=None):
    
    def myfunc(x, grad):
        #if p.istop != 0: raise nlopt.FORCED_STOP
        if grad.size > 0:
            grad[:]= p.df(x.copy())
        return asscalar(p.f(x))
    
    # TODO: add other local nlopt optimizers that cannot handle equality constraints
    if solver in [nlopt.LD_MMA] and (p.nbeq != 0 or p.nh != 0): 
        opt = nlopt.opt(nlopt.LD_AUGLAG, p.n)
        opt2 = nlopt.opt(solver, p.n)
        setStopCriteria(opt2, p, reduce=10.0)
        opt.set_local_optimizer(opt2)
    elif solver == nlopt.G_MLSL_LDS:
        opt = nlopt.opt(nlopt.G_MLSL_LDS, p.n)
        opt2 = nlopt.opt(nlopt.LD_TNEWTON_PRECOND_RESTART, p.n)
        setStopCriteria(opt2, p, reduce=10.0)
        opt.set_local_optimizer(opt2)        
    else:
        opt = nlopt.opt(solver, p.n)
    
    
    if opts is not None:
        for option, val in opts.items():
            getattr(opt, option)(val)
        
    opt.set_min_objective(myfunc)
    if any(p.lb==p.ub): p.pWarn('nlopt solvers badly handle problems with variables fixed via setting lb=ub')
    lb = [elem if isfinite(elem) else float(elem) for elem in p.lb.tolist()]
    ub = [elem if isfinite(elem) else float(elem) for elem in p.ub.tolist()]
    if any(isfinite(lb)): opt.set_lower_bounds(lb)
    if any(isfinite(ub)): opt.set_upper_bounds(ub)
    
    # TODO: A, Aeq
    
    if p.nc > 0:
        def c(result, x, grad):
            if grad.size > 0:
                grad[:] = p.dc(x.copy())
            result[:] = p.c(x)
        opt.add_inequality_mconstraint(c, array([p.contol]*p.nc).copy())
        
    if p.nb > 0:
        def c_lin(result, x, grad):
            if grad.size > 0:
                grad[:] = p.A.copy()
            result[:] = p._get_AX_Less_B_residuals(x)
        opt.add_inequality_mconstraint(c_lin, array([p.contol]*p.nb).copy())
    
    if p.nh > 0:
        def h(result, x, grad):
            if grad.size > 0:
                grad[:] = p.dh(x.copy())
            result[:] = p.h(x).copy()
        opt.add_equality_mconstraint(h, array([p.contol]*p.nh))

    if p.nbeq > 0:
        def h_lin(result, x, grad):
            if grad.size > 0:
                grad[:] = p.Aeq.copy()
            result[:] = p._get_AeqX_eq_Beq_residuals(x)
        opt.add_equality_mconstraint(h_lin, array([p.contol]*p.nbeq))
    
    setStopCriteria(opt, p)
    # others like fEnough, maxFunEvals, are handled by OO  kernel
    
    x0 = asfarray(p.x0).copy()
    
    #lb2 = copy(lb)
    #lb2[isinf(lb2)] = 0
    LB = copy(lb) #+ 1e-15*(ones(p.n) + abs(lb2))
    ind = x0 <= LB
    x0[ind] = LB[ind]
        
    #ub2 = copy(ub)
    #ub2[isinf(ub2)] = 0
    UB = copy(ub) #- 1e-15*(ones(p.n) + abs(ub2))
    ind = x0 >= UB
    x0[ind] = UB[ind]
    
    
    #x0[ind] = p.lb[ind] + 1e-15*abs((max(1.0, p.lb[x0<p.lb])))
    #x0[x0>p.ub] = p.ub[x0>p.ub] - 1e-15*min((1.0, abs(p.ub[x0>p.ub])))
#    if p.solver.__name__ == 'auglag':
#        opt.set_local_optimizer('mma')
    try:
        x = opt.optimize(x0.tolist()).copy()
        #if p.point(x).betterThan(p.point(p.xk)):
            #p.xk = x
    except:
        pass
    p.xk = p._bestPoint.x
        #x = p.xk
        #p.iterfcn(x)
    
    
    iStop = opt.get_stopval()
    
    if p.istop == 0:
        if iStop == nlopt.XTOL_REACHED:
            p.istop,  p.msg = SMALL_DELTA_X, '|| X[k] - X[k-1] || < xtol'
        elif iStop == nlopt.FTOL_REACHED:
            p.istop,  p.msg = SMALL_DELTA_F, '|| F[k] - F[k-1] || < ftol'
        else:
            p.istop = SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
#    p._opt = opt

def setStopCriteria(opt, p, reduce=1.0):
    opt.set_xtol_abs(p.xtol/reduce)
    opt.set_ftol_abs(p.ftol/reduce)
    opt.set_maxeval(p.maxFunEvals)
    #opt.set_maxiter(p.maxIter)
    if isfinite(p.maxTime): 
        opt.set_maxtime(p.maxTime)
    #    opt.set_xtol_rel(1e-1)
    #    opt.set_ftol_rel(1e-1)



__docformat__ = "restructuredtext en"

from numpy import *
from openopt import SLE

N = 1000
C = empty((N,N))
d =  1.5+80*sin(arange(N))

for j in xrange(N):
    C[j] = 8*sin(4.0+arange(j, N+j)**2) + 15*cos(j)

p = SLE(C, d)
#r = p.solve('defaultSLEsolver'), or just
r = p.solve()

print('max residual: %e' % r.ff)
#print('solution: %s' % r.xf)




from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F, MAX_NON_SUCCESS
#import numpy as np
from .tsp import main

class sa(baseSolver):
    __name__ = 'sa'
    __license__ = "BSD"
    __authors__ = "John Montgomery, connected to OO by Dmitrey"
    __alg__ = "simulated annealing"
    __license__ = 'Creative Commons Attribution 3.0 Unported'
    iterfcnConnected = True
    __homepage__ = ''
    #__optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'b', 'Aeq','beq','c','h']
    __info__ = ''
    

    # Check it!
    __isIterPointAlwaysFeasible__ = True

        
    def __solver__(self, p):
        
        #self.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        #self.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        #self.kernelIterFuncs.pop(MAX_NON_SUCCESS, None)
        
        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        p.kernelIterFuncs.pop(MAX_NON_SUCCESS, None)
        
        p._bestPoint = p.point(p.x0)
        p.solver._requiresBestPointDetection = True
        
        M = p.M
        iterations, score, best = main(M, p)
        
        p.ff = p.fk = score
        p.xk = p.xf = best
        if p.istop == 0: 
            p.istop = 1000
        



from numpy import dot, sign, zeros, all, isfinite, array, sqrt, any, isnan, pi, sin, arccos, inf, argmax, asfarray
import numpy
from numpy.linalg import norm

class DilationUnit():
    
    #vectorNorm = 0 # TODO: remove it?
    #dilationCoeff = 1.0
    maxScalarComponentsLength = 2
    
    
    def __init__(self, vector, dilationCoeff):
        #self.vectorDirection = None
        self.scalarComponents = []
        nv = norm(vector)
        assert nv != 0
        #self.vectorDirection, self.vectorNorm, self.dilationCoeff = vector/nv, nv, dilationCoeff
        self.vectorDirection, self.dilationCoeff = vector/nv, dilationCoeff
        
class Dilation():
    #maxUnits = 10
    #treshhold = 1.01
    th_phi = 0.1
    dilationCoeffThreshold = 0.999999
    prevRest = None
    #maxVectorNum = 50
    
    def __init__(self, maxUnitsNum): 
        self.maxUnitsNum = maxUnitsNum
        self.units = []
        self.unitsNum = 0
        self.T = numpy.float64
        if hasattr(numpy, 'float128'):
            pass
            self.T = numpy.float128
    
    def addDilationUnit(self, vector, dilationCoeff = 0.99999):
        assert all(isfinite(vector))
        self.unitsNum += 1
        v = self.T(vector.copy())
        nv = norm(v)
        v /= nv
        
        # TODO: COMMENT IT OUT
#        M = 0
#        for i, unit in enumerate(self.units):
#            M = max((M, abs(dot(unit.vectorDirection, v))))
#        if M > 1e-2: 
#            print 'warning: max M=', M, 'nv=', nv
#            return
        
        #self.units.add(DilationUnit(vector.copy(), dilationCoeff))
        self.units.append(DilationUnit(v, dilationCoeff))
        print 'add new dilation vector; curr num: ', len(self.units)
        
    def getProjections(self, vv):
        #assert len(self.units) != 0
        V = self.T(vv)
        NV = norm(V)
        V /= NV
        r= []
        #print 'norm(V):', norm(V)
        for unit in self.units:
            # TODO: try to involve less multiplication ops
            scalarComponent = dot(unit.vectorDirection, V)
#            print 'norm(unit.vectorDirection):', norm(unit.vectorDirection)
#            print 'scalarComponent>>>', scalarComponent
            component =  unit.vectorDirection * scalarComponent#V
            r.append((scalarComponent, component, unit))
        for scalarComponent, component, unit in r: 
            V -= component
        return r, V#*NV
    
    def getDilatedDirection(self, direction):
        projectionsInfo, rest = self.getProjections(direction)
        dilatedDirection = zeros(direction.size)
        for scalarComponent, component, unit in projectionsInfo:
            dilatedDirection += component * unit.dilationCoeff
        return projectionsInfo, dilatedDirection+rest

    def getRestrictedDilatedDirection(self, direction):
        projectionsInfo, rest = self.getProjections(direction)
        dilatedDirection = zeros(direction.size)
        s, ns = [], []
        for scalarComponent, component, unit in projectionsInfo:
            t = component * unit.dilationCoeff
            s.append(t)
            ns.append(norm(t))
            dilatedDirection += t
        r = dilatedDirection+rest
        nr = norm(r)
        for i in xrange(len(s)):
            if ns[i] < 1e-10*nr:
                r += 1e-10*nr*s[i]/ns[i]-s[i]
        return projectionsInfo, r

    def getMostInsufficientUnit(self, scalarComponents):
        assert self.unitsNum != 0 
        ind, miUnit, miValue = 0, self.units[0], self.units[0].dilationCoeff#abs(scalarComponents[0])*(1-self.units[0].dilationCoeff)
        for i, unit in enumerate(self.units):
            #newValue = unit.dilationCoeff*abs(scalarComponents[i])
            newValue = unit.dilationCoeff#abs(scalarComponents[i])*(1-unit.dilationCoeff)
            if newValue > miValue:
                ind, miUnit, miValue = i, unit, newValue
        return ind, miUnit
        
    def updateDilationCoeffs2(self, scalarComponents, rest):
        arr_u = array([unit.dilationCoeff for unit in self.units])
        if self.unitsNum == 1: 
            self.units[0].dilationCoeff /= 2.0
            return
        m = self.unitsNum
        n = rest.size
        #th = norm(rest) * sqrt(n-m)
        
        for i, unit in enumerate(self.units):
            c = unit.dilationCoeff * abs(scalarComponents[i]) / n
            
            if c < 0.125:
                unit.dilationCoeff  *= 2.0
            elif c > 0.25 :
                unit.dilationCoeff  /= 2.0
            print i, unit.dilationCoeff
            
            
    def updateDilationCoeffs(self, scalarComponents, rest):
        arr_u = array([unit.dilationCoeff for unit in self.units])
#        if self.unitsNum == 1: 
#            self.units[0].dilationCoeff /= 2.0
#            return
        Ui2 = arr_u ** 2
        UiSCi = abs(array([unit.dilationCoeff*scalarComponents[i] for i, unit in enumerate(self.units)]))
        Ui2SCi2 = array(UiSCi) ** 2
        S, S2 = sum(Ui2), sum(Ui2SCi2)
        SCi = abs(array(scalarComponents))
        SCi2 = SCi ** 2
        
        alp = 2.0
        beta = 1.0 / alp
        
        #k = sqrt(S2 / (alp*sum((1.0-UiSCi)**2 * UiSCi2)))
        #rr = k * sqrt(1.0-UiSCi)**2
        m, n = self.unitsNum, rest.size
        b = abs(beta)
        #b = min((abs(beta), m/(16*n*sqrt(sum(UiSCi)))))
        #b = m/(n*sqrt(sum(UiSCi)))
        
        nr2 = norm(rest) ** 2
        k = b*sqrt(S2 / sum(Ui2SCi2*(1.0-UiSCi)))
        #k = sqrt(((b2-1)*nr2 + b2 * S2) / sum(Ui2SCi2*(1.0-UiSCi)))
#        k1 = sqrt(b2 * S2 / sum(Ui2SCi2*(1.0-UiSCi)))
#        m, n = self.unitsNum, rest.size
#        rr1 = k1 * (1-UiSCi)
#        u1 = rr1 * arr_u
        
        
        
        rr = k * (1-UiSCi)
        assert k > 0
        

        #k = sqrt(S2 / (alp*sum((1.0-SCi)**2 * Ui2SCi2)))
        #rr = k * sqrt(1.0-SCi)**2
       
        rr[rr>4.0] = 4.0
        rr[rr<0.25] = 0.25
        r = rr * arr_u
        #r[r<1e-20] = 1e-20
        assert len(r) == self.unitsNum == len(self.units)
        #print '--------------------'
        
        for i, unit in enumerate(self.units):
            unit.dilationCoeff = r[i]
        print 'nU=%d k=%0.1g r_min=%0.1g r_max=%0.1g' % (self.unitsNum, k, min(r), max(r))
        #print r
            #print i, unit.dilationCoeff
#        print 'old sum:', S2,'new sum:', sum(array([unit.dilationCoeff*scalarComponents[i] for i, unit in enumerate(self.units)])**2)
#        print '====================='
   
    def _updateDilationInfo(self, _dilationDirection_, ls, _moveDirection_):
        r = {'increased':0, 'decreased':0}
        #projectionsInfo, dilatedDirectionComponent, rest = self.getDilatedDirection(_moveDirection_)
        
        projectionsInfo1, rest1 = self.getProjections(_dilationDirection_)
        
        print 'norm(rest1):', norm(rest1)
        #cond_add = norm(rest1) > 1e-2
        s = abs(asfarray([scalarComponent for (scalarComponent, component, unit) in projectionsInfo1]))
        #cond_add = self.unitsNum == 0 or any(norm(rest1) > 64.0*asfarray([unit.dilationCoeff * s[i] for i, unit in enumerate(self.units)]))
        cond_add = norm(rest1) > 1e-3 #or (self.prevRest is not None and dot(self.prevRest, rest1) <= 0)
        if cond_add: 
            self.addDilationUnit(rest1)
            projectionsInfo1, rest1 = self.getProjections(_dilationDirection_)
            print 'norm(rest11):', norm(rest1)
        
        #print '!>', dot(dilatedDirectionComponent1, rest1) / norm(dilatedDirectionComponent1) / norm(rest1)
        #print '!!>', dilatedDirectionComponent1, rest1
        #assert norm(dilatedDirectionComponent1) > 1e-10
        
#        mostUnusefulUnitNumber = -1
#        mostUnusefulUnitCoeff = -1
#        for i, u in enumerate(self.units):
#            if u.dilationCoeff > mostUnusefulUnitCoeff:
#                mostUnusefulUnitNumber, mostUnusefulUnitCoeff = i, u.dilationCoeff
        
        #print 'norm(_dilationDirection_) :', norm(_dilationDirection_) 
        #s = norm(rest1) / norm(dilatedDirectionComponent1)       
        #print 's:', s
        
        scalarComponents = [scalarComponent for (scalarComponent, component, unit) in projectionsInfo1]
        #print argmax(scalarComponents)
        
        #m = len(self.units)
        #n = _dilationDirection_.size
        
        #print 'norm(rest1), norm(scalarComponents, inf):', norm(rest1) ,  norm(scalarComponents, inf)
        #print ''
        #condReasonableBigRest = self.unitsNum == 0  or norm(rest1) > norm(scalarComponents, inf)#miUnit.dilationCoeff* abs(scalarComponents[ind_mi])   # s > 0.05
        #print 'norm(rest1):', norm(rest1), 'miUnit.dilationCoeff:', miUnit.dilationCoeff, 'miUnit.sc:', scalarComponents[ind_mi]
        #if 1 or norm(rest1) > 0.9:
            #print '>', rest1/norm(rest1), norm(rest1), [Unit.dilationCoeff for Unit in self.units], [Unit.vectorDirection for Unit in self.units]
        #and self.prevRest is not None and (True or dot(self.prevRest, rest) <= 0)
        
        
        #projectionsInfo2, dilatedDirectionComponent2, rest2 = self.getDilatedDirection( dilatedDirectionComponent1 + rest1)
        #assert norm(dilatedDirectionComponent2) > 1e-10
        #projectionsInfo3, dilatedDirectionComponent, rest = self.getDilatedDirection( dilatedDirectionComponent + rest)
        
        projectionsInfo,  rest = projectionsInfo1, rest1
        #print 'norm(r1), norm(d1):', norm(rest1), norm(dilatedDirectionComponent1)
        #abs_tan_phi = norm(rest1) / norm(dilatedDirectionComponent1)
    
        #projectionsInfo,  dilatedDirectionComponent, rest = projectionsInfo2, dilatedDirectionComponent2, rest2
        #print 'norm(r2), norm(d2):', norm(rest2), norm(dilatedDirectionComponent2)
        
        #cond_drift = self.prevRest is not None and dot(self.prevRest, rest1) > 0
        # TODO: what if self.prevRest is None?
        
        #haveToAdd = condReasonableBigRest and any(rest1!=0) #and not cond_drift



            
        self.updateDilationCoeffs(scalarComponents, rest)



        self.prevRest = rest.copy()
        #print 'self.unitsNum, self.maxUnitsNum:', self.unitsNum,  self.maxUnitsNum
        if self.unitsNum >= self.maxUnitsNum:
            self.unitsNum = 0
            self.units = []
#            ind_mi, miUnit = self.getMostInsufficientUnit(scalarComponents)
#            self.units.pop(ind_mi)
#            self.unitsNum -= 1

#            for unit in self.units:
#                unit.dilationCoeff /= miUnit.dilationCoeff
            #print 'mi removed:', ind_mi
        nRemoved = self.cleanUnnessesaryDilationUnits()     
        if nRemoved: print 'nRemoved:', nRemoved
        return r            
            #d = 1.0
        
        #projectionsInfo, dilatedDirectionComponent, rest = self.getDilatedDirection(_dilationDirection_)
        
#        for scalarComponent, component, unit in projectionsInfo:
#            # angle between ort and dilation direction
#            angle = arccos(scalarComponent) # values from 0 to pi
#            if angle > pi/2: angle = pi - angle
#            if d < 1.0: 
#                unit.dilationCoeff *= max((0.5, sin(d*angle)))
#            elif d > 1.0:
#                unit.dilationCoeff *= max((2.0, d/sqrt(1-scalarComponent**2)))
#
#        if cond_overdilated:
#            #TODO - omit repeated calculations
#            nRemoved = self.cleanUnnessesaryDilationUnits()        
#            print 'REMOVED: ', nRemoved#, 'increaseMultiplier:', increaseMultiplier



#            if sign(dot(_moveDirection_, component)) == sign(scalarComponent):
#                #print 'case 1'
#                unit.dilationCoeff *=  multiplier
#                if unit.dilationCoeff  > 1.0: unit.dilationCoeff  = 1.0
#            else:
#                reduceMultiplier = max((0.5, sqrt(1 - scalarComponent**2)))
#                unit.dilationCoeff *= reduceMultiplier
                #unit.dilationCoeff /=  2.0#multiplier
                #print 'case 2'

        
        
        
#        #cond_overDilated = abs_tan_phi > 0.5#min((2.0, 1.0 / self.th_phi))
#        if abs_tan_phi < self.th_phi: #abs_tan_phi  < 0.1 * len(self.units) / (_dilationDirection_.size-len(self.units)): 
#            koeff = 0.5
#            for scalarComponent, component, unit in projectionsInfo:
#                unit.dilationCoeff *=  koeff * max((0.1, sqrt(1 - scalarComponent**2)))
#        #elif cond_overDilated: # TODO: use separate parameter instead of (1.0 / self.th_phi)
#        else:
#        #elif abs_tan_phi > 
#            multiplier = self.th_phi / abs_tan_phi
#            if multiplier > 2.0: multiplier = 2.0
#            elif multiplier < 1.3: multiplier = 1.3           
#            for scalarComponent, component, unit in projectionsInfo:
#                if sign(dot(_moveDirection_, component)) == sign(scalarComponent):
#                    unit.dilationCoeff *=  multiplier
#                    #pass
                
        
#        for scalarComponent, component, unit in projectionsInfo:
#            pass
            #reduceMultiplier = max((0.25, sqrt(1 - scalarComponent**2)))
            #unit.dilationCoeff *= reduceMultiplier        

        ##########################################
        
        
        # get NEW rest
        # TODO - omit repeated calculations
        #projectionsInfo, dilatedDirectionComponent, rest = self.getDilatedDirection(_dilationDirection_)
        ##########################################
        
        #if self.prevRest is not None:print 'sign dot:', sign(dot(self.prevRest, rest))
        #print 'norm(rest) / norm(dilatedDirectionComponent:', norm(rest)/ norm(dilatedDirectionComponent)
        
        #haveToAdd = True
        



    def cleanUnnessesaryDilationUnits(self):
        indUnitsToRemove = []
        for i, unit in enumerate(self.units):
            if unit.dilationCoeff > self.dilationCoeffThreshold:
                print '>>', unit.dilationCoeff ,  self.dilationCoeffThreshold
                indUnitsToRemove.append(i)
                #unitsToRemove.add(unit)
        for j in xrange(len(indUnitsToRemove)):
            self.units.pop(indUnitsToRemove[-1-j])
        nRemoved = len(indUnitsToRemove)
        self.unitsNum -= nRemoved
        #if nRemoved!=0: print 'dilation units: removed = %d left=%d' %  (nRemoved, len(self.units))
        return nRemoved
        
#    def cleanUnnessesaryDilationUnits(self):
#        unitsToRemove = set()
#        for unit in self.units:
#            if unit.dilationCoeff >= self.treshhold:
#                unitsToRemove.add(unit)
#        self.units = self.units.difference(unitsToRemove)


#    def updateDilationInfo(self, projectionsInfo, ls):
#        r = {'increased':0, 'decreased':0}
#        
#        for scalarComponent, component, unit in projectionsInfo:
#            scs = unit.scalarComponents
#            if len(scs) >= unit.maxScalarComponentsLength:
#                scs.pop(0)
#            scs.append(scalarComponent)
#            signs = sign(scs)
#            
#            #cond_1 = norm(unit.vectorDirection - array([1.0]+[0]*(unit.vectorDirection.size-1)))<1e-4
#            #cond_1 = any(unit.vectorDirection[:10]==1)
#            
#            # TODO: remove cycles
#            
#            if len(signs)>1: # TODO: handle signs[-1-i] == 0 case
#                if ls <= 1:
#                    if signs[-1] == signs[-2]:
#                        pass
#                    else:
#                        unit.dilationCoeff /= 2.0
#                        r['decreased'] += 1
#                elif ls > 1:
#                    if signs[-1] == signs[-2]:
#                        unit.dilationCoeff *= 1 + 0.1#ls/3.0
#                        r['increased'] += 1
#                    else:
#                        pass
#        return r



from baseProblem import MatrixProblem
from numpy import  dot, asfarray, zeros, asscalar
import numpy as np


class SOCP(MatrixProblem):
    probType = 'SOCP'
    _optionalData = ['d', 'A', 'Aeq', 'b', 'beq', 'lb', 'ub']
    goal = 'minimum'
    allowedGoals = ['minimum', 'min']
    #TODO: add goal=max, maximum
    showGoal = True
    #contol = 1e-6
    
    expectedArgs = ['f', 'C']
    # required are f, C, d for OO and f for FD
    
    FuncDesignerSign = 'f'
    
    def __init__(self, *args, **kwargs):
        MatrixProblem.__init__(self, *args, **kwargs)
        if self._isFDmodel():
            self.x0 = self.C
            return
        self.f = asfarray(self.f)
        self.n = self.f.size # for p.n to be available immediately after assigning prob
        if self.x0 is None: self.x0 = zeros(self.n)
    
    def _Prepare(self):
        MatrixProblem._Prepare(self)
        if self._isFDmodel():
            C, d, q, s = renderFDmodel(self)        
            self.C, self.d, self.q, self.s = C, d, q, s
            D = self.f.D(self._x0, fixedVars = self.fixedVars)
            _f = self._point2vector(D).flatten()
            self.f, self._f = _f, self.f

    def __finalize__(self):
        MatrixProblem.__finalize__(self)
        if self.goal in ['max', 'maximum']:
            self.f = -self.f
            for fn in ['fk', ]:#not ff - it's handled in other place in RunProbSolver.py
                if hasattr(self, fn):
                    setattr(self, fn, -getattr(self, fn))
        if hasattr(self, '_f'):
            self.f = self._f
            
    def objFunc(self, x):
        return asscalar(dot(self.f, x))

def renderFDmodel(p):
    from FuncDesigner import hstack, ooarray
    C, d, q, s = [], [], [], []
    D_kwargs = p._D_kwargs
    Z = p._Z
    D_kwargs_y  = D_kwargs.copy()
    D_kwargs_y['useSparse'] = False
    D_kwargs_y['exactShape'] = False
    for c in p.constraints:
        order = c.oofun.getOrder(p.freeVarsSet, p.fixedVarsSet, fixedVarsScheduleID = p._FDVarsID)
        if order < 2:
            continue # linear or box-bound constraint
        if c.descriptor is None:
            p.err('incorrect constraint for SOCP')
        x, op, y = c.descriptor
        if op == '>':
            x, y = y, x
        y_order = 0 if np.isscalar(y) or type(y) == np.ndarray else y.getOrder(p.freeVarsSet, p.fixedVarsSet, fixedVarsScheduleID = p._FDVarsID)
        if y_order > 1 or x.engine != 'norm2':
            p.err('incorrect constraint for SOCP')
        assert len(x.input) == 1, 'incorrect constraint for SOCP'
        z = x._norm_arg
        if isinstance(z, ooarray):
            z = hstack(z)
        
        d.append(z(Z))
        D = z.D(Z, **D_kwargs)
        C.append(p._pointDerivative2array(D))      
        
        S = y if y_order == 0 else y(Z)
        s.append(S)
        if y_order > 0:
            D = y.D(Z, **D_kwargs_y)
            rr = p._pointDerivative2array(D,useSparse = False).flatten()
        else:
            rr = np.zeros(p.n)
        q.append(rr)
    return C, d, q, s













from numpy import ceil, floor, argmax, ndarray, copy, arange
from setDefaultIterFuncs import SMALL_DELTA_X, SMALL_DELTA_F
from LP import LP

class MILP(LP):
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'intVars', 'boolVars']
    probType = 'MILP'
    expectedArgs = ['f', 'x0']
    allowedGoals = ['minimum', 'min', 'max', 'maximum']
    showGoal = True
    _milp_prepared = False
    
    def _Prepare(self):
        if self._milp_prepared: return
        self._milp_prepared = True
        LP._Prepare(self)
        r = []
        if type(self.intVars) not in [list, tuple, set] and not isinstance(self.intVars, ndarray):
            self.intVars = [self.intVars]
        if self.isFDmodel:
            
            ########### obsolete, to be removed in future versions
            if self.intVars != []:
                s = '''
                For FuncDesigner models prob parameter intVars is deprecated
                use oovar(..., domain = int) instead'''
                self.err(s)
#            for iv in self.intVars:
#                if self.fixedVars is not None and iv in self.fixedVars or\
#                self.freeVars is not None and iv not in self.freeVars:
#                    continue
#                r1, r2 = self._oovarsIndDict[iv]
#                r += range(r1, r2)
            ########### obsolete, to be removed in future versions
            
            for v in self._freeVarsList:
                if isinstance(v.domain, (tuple, list, ndarray, set)):
                    self.err('for FuncDesigner MILP models only variables with domains int, bool or None (real) are implemented for now')
                if v.domain is int or v.domain is  'int' or v.domain is bool or v.domain is 'bool':
                    r1, r2 = self._oovarsIndDict[v]
                    r += arange(r1, r2).tolist()
            
            self.intVars, self._intVars = r, self.intVars
        self._intVars_vector = self.intVars
                
        if SMALL_DELTA_X in self.kernelIterFuncs: self.kernelIterFuncs.pop(SMALL_DELTA_X)
        if SMALL_DELTA_F in self.kernelIterFuncs: self.kernelIterFuncs.pop(SMALL_DELTA_F)
        def getMaxResidualWithIntegerConstraints(x, retAll = False):
            r, fname, ind = self.getMaxResidual2(x, True)
            if len(self.intVars) != 0:
                intV = x[self.intVars]
                intDifference = abs(intV-intV.round())
                intConstraintNumber = argmax(intDifference)
                intConstraint = intDifference[intConstraintNumber]
                if intConstraint > r:
                    intConstraintNumber = self.intVars[intConstraintNumber]
                    r, fname, ind = intConstraint, 'int', intConstraintNumber 
            if retAll:
                return r, fname, ind
            else:
                return r
        self.getMaxResidual, self.getMaxResidual2 = getMaxResidualWithIntegerConstraints, self.getMaxResidual
        
        # TODO: 
        # 1) ADD BOOL VARS
        self.lb, self.ub = copy(self.lb), copy(self.ub)
        self.lb[self.intVars] = ceil(self.lb[self.intVars])
        self.ub[self.intVars] = floor(self.ub[self.intVars])
        
#        if self.goal in ['max', 'maximum']:
#            self.f = -self.f
    def __finalize__(self):
        LP.__finalize__(self)
        if self.isFDmodel: self.intVars = self._intVars



import numpy as np
try:
    from scipy.sparse import find, isspmatrix
except ImportError:
    isspmatrix = lambda *args, **kw: False


import socket, subprocess

#from time import sleep
TCP_IP = '127.0.0.1'
TCP_PORT = 5001

from os.path import abspath, dirname, normpath
path = dirname(abspath(__file__))

def wh(d, matlabExecutable):
    args = ['-nodesktop', 
                   '-r',  '"addpath %s ; OpenOpt_Proxy"' %normpath(path)
                   ]
    Matlab = matlabExecutable+' '+' '.join(args)
    subprocess.Popen(Matlab, shell=True)
    for k, v in d.items():
        exec(k + '=v')
#    is_sparse = False # to suppress Python3 issue
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind((TCP_IP, TCP_PORT))
    s.listen(1)

    #print "waiting for connection"
    conn, addr = s.accept() # hangs until other end connects
    #print 'Connection address:', addr

    isPyPy = False

    CycleCond = True

    while CycleCond:
    #    print "waiting for message"
        cmd_class = conn.recv(10).strip()
    #    print "cmd_class: ",cmd_class
        if cmd_class in ('put', b'put'):
            name = conn.recv(10).strip()
#            if type(name) == bytes:
#                name = name.decode()
    #        print "name: ",name
            shape_str = conn.recv(30).strip()
    #        print "shape_str: ",shape_str
            shape = tuple(map(int,shape_str.split()))
            n_floats = np.prod(shape)
            targ_flat = np.zeros(n_floats,'float')        
            n_read = 0
            while n_read < n_floats:
                n_toread = min(128,n_floats-n_read)
                arr = np.fromstring(conn.recv(n_toread*8),dtype='>f8')
                targ_flat[n_read:n_read+n_toread] = arr
                n_read += n_toread
    #            print "%i/%i floats read"%(n_read,n_floats)        
    #        print('isPyPy1:', isPyPy)
            if isPyPy:
                targ = targ_flat.reshape(shape)#[::-1]).T
            else:
                targ = targ_flat.reshape(shape,order="F")
    #        print('targ:', targ)
    #        input()
    #        print "received array: ",targ
            exec("%s = targ"%name)
        elif cmd_class in ('exec', b'exec'):
            stmt = conn.recv(100).strip()
#            if type(stmt) == bytes:
#                stmt = stmt.decode()
    #        print "statment: ",stmt
            exec(stmt)
        elif cmd_class in ("get", b'get'):
            name = conn.recv(10).strip()
#            name_str = name.decode() if type(name) == bytes else name
                
    #        print "getting variable: ",name
            try:
#                print(name, type(name))
#                print(name == 'is_sparse')
#                print(type(is_sparse))

#                if name_str == 'is_sparse':
#                    src = is_sparse
#                else:
                exec("src = %s"%name)
            except NameError:
                exec("src = []")
#            except UnboundLocalError:
#                exec("src = []")
            src = np.atleast_2d(src)
            Str = "%30s"%str(src.shape).replace(")","]").replace("(","[")
#            print(Str)
            conn.send(Str)
            if isPyPy:
                src_str = src.astype('>f8').T.tostring()
            else:
                src_str = src.astype('>f8').tostring(order="F")
            n_floats = src.size
            n_sent = 0
            while n_sent < n_floats:
                n_tosend = min(1024,n_floats-n_sent)            
                conn.send(src_str[n_sent*8:(n_sent+n_tosend)*8])
                n_sent += n_tosend
    #            print "%i/%i floats sent"%(n_sent,n_tosend)  
        else:
            raise Exception("unrecognized command (cmd_class=%s)"%cmd_class)



from numpy import asarray,  ones, nan, concatenate, array
from openopt.kernel.ooMisc import xBounds2Matrix
from cvxopt_misc import *
import cvxopt.solvers as cvxopt_solvers
from cvxopt.base import matrix
from openopt.kernel.setDefaultIterFuncs import SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON,  IS_MAX_ITER_REACHED, IS_MAX_TIME_REACHED, FAILED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON, UNDEFINED
#import os

def CVXOPT_LP_Solver(p, solverName):
    #os.close(1); os.close(2)
    if solverName == 'native_CVXOPT_LP_Solver': solverName = None
    cvxopt_solvers.options['maxiters'] = p.maxIter
    cvxopt_solvers.options['feastol'] = p.contol    
    cvxopt_solvers.options['abstol'] = p.ftol
    if p.iprint <= 0:
        cvxopt_solvers.options['show_progress'] = False
        cvxopt_solvers.options['LPX_K_MSGLEV'] = 0
        cvxopt_solvers.options['MSK_IPAR_LOG'] = 0
        if solverName == 'glpk': 
            from cvxopt import glpk  
            glpk.options['LPX_K_MSGLEV'] = 0  

    xBounds2Matrix(p)
    #WholeRepr2LinConst(p)

    # CVXOPT have some problems with x0 so currently I decided to avoid using the one
    #if  p.x0.size>0 and p.x0.flatten()[0] != None and all(isfinite(p.x0)):
    #    sol= cvxopt_solvers.solvers.lp(Matrix(p.f), Matrix(p.A), Matrix(p.b), Matrix(p.Aeq), Matrix(p.beq), solverName)
    #else:

    if (len(p.intVars)>0 or len(p.binVars)>0) and solverName == 'glpk':
        from cvxopt.glpk import ilp
        c = Matrix(p.f)
        A, b = Matrix(p.Aeq),  Matrix(p.beq)
        G, h = Matrix(p.A),  Matrix(p.b)
        if A is None:
            A = matrix(0.0,  (0, p.n))
            b = matrix(0.0,(0,1))
        if G is None:
            G = matrix(0.0,  (0, p.n))
            h = matrix(0.0,(0,1))

        (status, x) = ilp(c, G, h, A, b, set(p.intVars), B=set(p.binVars))
        if status == 'optimal': p.istop = SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
        elif status == 'maxiters exceeded': p.istop = IS_MAX_ITER_REACHED
        elif status == 'time limit exceeded': p.istop = IS_MAX_TIME_REACHED
        elif status == 'unknown': p.istop = UNDEFINED
        else: p.istop = FAILED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
        if x is None:
            p.xf = nan*ones(p.n)
        else:
            p.xf = array(x).flatten()#w/o flatten it yields incorrect result in ff!
        p.ff = sum(p.dotmult(p.f, p.xf))
        p.msg = status
    else:
#        if len(p.b) != 0: 
#            s0 = matrix(p.b - dot(p.A, p.x0))
#        else:
#            s0 = matrix()
#        primalstart = {'x': matrix(p.x0), 's': s0}
#        sol = cvxopt_solvers.lp(Matrix(p.f), Matrix(p.A), Matrix(p.b), Matrix(p.Aeq), Matrix(p.beq), solverName, primalstart)
        sol = cvxopt_solvers.lp(Matrix(p.f), Matrix(p.A), Matrix(p.b), Matrix(p.Aeq), Matrix(p.beq), solverName)
        p.msg = sol['status']
        if p.msg == 'optimal' :  p.istop = SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
        else: p.istop = -100
        if sol['x'] is not None:
            p.xf = asarray(sol['x']).flatten()
            # ! don't involve p.ff  - it can be different because of goal and additional constant from FuncDesigner
            p.duals = concatenate((asarray(sol['y']).flatten(), asarray(sol['z']).flatten()))
        else:
            p.ff = nan
            p.xf = nan*ones(p.n)



from numpy import  inf, any, copy, dot, where, all, nan, isfinite, float64, isnan,  \
max, sign, array_equal, matrix, delete, ndarray, asfarray, argmax
from numpy.linalg import norm#, solve, LinAlgError

from openopt.kernel.baseSolver import *
#from openopt.kernel.Point import Point
from openopt.kernel.setDefaultIterFuncs import *
#from openopt.solvers.UkrOpt.UkrOptMisc import getBestPointAfterTurn
from openopt.solvers.UkrOpt.PolytopProjection import PolytopProjection

class gsubg(baseSolver):
    __name__ = 'gsubg'
    __license__ = "BSD"
    __authors__ = "Dmitrey"
    __alg__ = "Nikolay G. Zhurbenko generalized epsilon-subgradient"
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    iterfcnConnected = True
    _canHandleScipySparse = True
    _requiresBestPointDetection = True

    #gsubg default parameters
    h0 = 1.0
    hmult = 0.5
    T = float64
    
    showLS = False
    show_hs = False
    showRes = False
    show_nnan = False
    doBackwardSearch = True
    new_bs = True
    
    maxVectors = 100
    maxShoots = 15
    sigma = 1e-3
    
    dual = True
    addASG = False
    
    # TODO: involve it?
    useLinePoints = True

    def __init__(self): 
        self.approach = 'all active'
        self.qpsolver = 'cvxopt_qp'
        self.ls_direction = 'simple'
        self.dilation = 'auto'
        
    def __solver__(self, p):
        assert self.approach == 'all active'
        #if not p.isUC: p.warn('Handling of constraints is not implemented properly for the solver %s yet' % self.__name__)
        
        dilation = self.dilation
        assert dilation in ('auto', True, False, 0, 1)
        if dilation == 'auto': 
            dilation = False
            #dilation = True if p.n < 150 else False
            p.debugmsg('%s: autoselect set dilation to %s' %(self.__name__, dilation))
        if dilation:
            from Dilation import Dilation
            D = Dilation(p)
        
#        LB, UB = p.lb, p.ub
#        fin_lb = isfinite(LB)
#        fin_ub = isfinite(UB)
#        ind_lb = where(fin_lb)[0]
#        ind_ub = where(fin_ub)[0]
#        ind_only_lb = where(logical_and(fin_lb, logical_not(fin_ub)))[0]
#        ind_only_ub = where(logical_and(fin_ub, logical_not(fin_lb)))[0]
#        ind_bb = where(logical_and(fin_ub, fin_lb))[0]
#        lb_val = LB[ind_only_lb]
#        ub_val = UB[ind_only_ub]
#        dist_lb_ub = UB[ind_bb] - LB[ind_bb]
#        double_dist_lb_ub = 2 * dist_lb_ub
#        ub_bb = UB[ind_bb]
#        doubled_ub_bb = 2 * ub_bb
#        def Point(x):
#            z = x.copy()
#            z[ind_only_lb] = abs(x[ind_only_lb]-lb_val) + lb_val
#            z[ind_only_ub] = ub_val - abs(x[ind_only_ub]-ub_val) 
#            
#            ratio = x[ind_bb] / double_dist_lb_ub
#            z1 = x[ind_bb] - array(ratio, int) * double_dist_lb_ub
#            ind = where(z1>ub_bb)[0]
#            z1[ind] = doubled_ub_bb - z1[ind]
#            z[ind_bb] = z1
#            #raise 0
#            return p.point(z)
        Point = lambda x: p.point(x)
        
        h0 = self.h0

        T = self.T
        # alternatively instead of alp=self.alp etc you can use directly self.alp etc

        #n = p.n
        x0 = p.x0
        
        if p.nbeq == 0 or any(abs(p._get_AeqX_eq_Beq_residuals(x0))>p.contol): # TODO: add "or Aeqconstraints(x0) out of contol"
            x0[x0<p.lb] = p.lb[x0<p.lb]
            x0[x0>p.ub] = p.ub[x0>p.ub]
        
        hs = asarray(h0, T)
        #ls_arr = []

        """                         Nikolay G. Zhurbenko generalized epsilon-subgradient engine                           """
        bestPoint = Point(asarray(copy(x0), T))
        bestFeasiblePoint = None if not bestPoint.isFeas(True) else bestPoint
        prevIter_best_ls_point = bestPoint
        best_ls_point = bestPoint
        iterStartPoint = bestPoint
        #prevIter_bestPointAfterTurn = bestPoint
        bestPointBeforeTurn = None
        g = bestPoint._getDirection(self.approach)
        g1 = iterStartPoint._getDirection(self.approach, currBestFeasPoint = bestFeasiblePoint)
        if not any(g) and all(isfinite(g)):
            # TODO: create ENUMs
            p.istop = 14 if bestPoint.isFeas(False) else -14
            p.msg = 'move direction has all-zero coords'
            return

        HS = []
        LS = []
        
        # TODO: add possibility to handle f_opt if known instead of fTol
        #fTol = 1.0
        if p.fTol is None:
            p.warn("""The solver requres user-supplied fTol (objective function tolerance); 
            since you have not provided it value, 15*ftol = %0.1e will be used""" % (15*p.ftol))
            p.fTol = 15 * p.ftol
        fTol_start = p.fTol/2.0
        fTol = fTol_start
        
        subGradientNorms, points, values, isConstraint, epsilons, inactive, normedSubGradients, normed_values = [], [], [], [], [], [], [], []
        StoredInfo = [subGradientNorms, points, values, isConstraint, epsilons, inactive, normedSubGradients, normed_values]
        nMaxVec = self.maxVectors
        nVec = 0
        ns = 0
        #ScalarProducts = empty((10, 10))
        maxQPshoutouts = 15
        
        
        """                           gsubg main cycle                                    """
        itn = -1
        while True:
            itn += 1
            # TODO: change inactive data removing 
            # TODO: change inner cycle condition
            # TODO: improve 2 points obtained from backward line search
            koeffs = None
            
            while ns < self.maxShoots:
                
                ns += 1
                nAddedVectors = 0
                projection = None
                F0 = asscalar(bestFeasiblePoint.f() - fTol_start) if bestFeasiblePoint is not None else nan
                
                #iterStartPoint = prevIter_best_ls_point
                if bestPointBeforeTurn is None:
                    sh = schedule = [bestPoint]
                    #x = iterStartPoint.x
                else:
                    sh = [point1, point2] if point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) else [point2, point1]
                    #sh = [iterStartPoint, bestPointBeforeTurn, bestPointAfterTurn]
                    #sh.sort(cmp = lambda point1, point2: -1+2*int(point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint)))
                    iterStartPoint = sh[-1]
                    schedule = [point for point in sh if id(point.x) != id(points[-1])]
                    #x = iterStartPoint.x.copy()
                    #x = 0.5*(point1.x+point2.x) 
                #print 'len(schedule):', len(schedule)
                
                
                x = iterStartPoint.x.copy()
                #print 'itn:', itn, 'ns:', ns, 'x:', x, 'hs:', hs
#                if itn != 0:
#                    Xdist = norm(prevIter_best_ls_point.x-bestPointAfterTurn.x)
#                    if hs < 0.25*Xdist :
#                        hs = 0.25*Xdist
                
                
                iterInitialDataSize = len(values)
                for point in schedule:
                    if (point.sum_of_all_active_constraints()>p.contol / 10 or not isfinite(point.f())) and any(point.sum_of_all_active_constraints_gradient()):
                        #print '111111'
#                    if not point.isFeas(True):
                        # TODO: use old-style w/o the arg "currBestFeasPoint = bestFeasiblePoint"
                        #tmp = point._getDirection(self.approach, currBestFeasPoint = bestFeasiblePoint)
                        nVec += 1
                        tmp = point.sum_of_all_active_constraints_gradient()#, currBestFeasPoint = bestFeasiblePoint)
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix):
                            tmp = tmp.A.flatten()
                        n_tmp = norm(tmp)
                        assert n_tmp != 0.0
                        normedSubGradients.append(tmp/n_tmp)
                        subGradientNorms.append(n_tmp)
                        val = point.sum_of_all_active_constraints()
                        values.append(asscalar(val))
                        normed_values.append(asscalar(val/n_tmp))
                        #epsilons.append(asscalar(val / n_tmp - dot(point.x, tmp)/n_tmp**2))
                        epsilons.append(asscalar((val + dot(point.x, tmp))/n_tmp))
                        #epsilons.append(asscalar(val - dot(point.x, tmp))/n_tmp)
                        #epsilons.append(asscalar(val))
                        isConstraint.append(True)
                        points.append(point.x)
                        inactive.append(0)
                        nAddedVectors += 1                    
                    if bestFeasiblePoint is not None and isfinite(point.f()):
                        #print '222222'
                        tmp = point.df()
                        if not isinstance(tmp, ndarray) or isinstance(tmp, matrix):
                            tmp = tmp.A
                        tmp = tmp.flatten()
                        n_tmp = norm(tmp)
                        if n_tmp < p.gtol:
                            p._df = n_tmp # TODO: change it 
                            p.iterfcn(point)
                            return
                        nVec += 1
                        normedSubGradients.append(tmp/n_tmp)
                        subGradientNorms.append(n_tmp)
                        val = point.f()
                        values.append(asscalar(val))
                        normed_values.append(asscalar(val/n_tmp))
                        epsilons.append(asscalar((val + dot(point.x, tmp))/n_tmp))
                        isConstraint.append(False)
                        points.append(point.x)
                        inactive.append(0)
                        nAddedVectors += 1       
                if self.addASG and itn != 0 and Projection is not None:
                    tmp = Projection
                    if not isinstance(tmp, ndarray) or isinstance(tmp, matrix):
                        tmp = tmp.A
                    tmp = tmp.flatten()
                    n_tmp = norm(tmp)

                    nVec += 1
                    normedSubGradients.append(tmp/n_tmp)
                    subGradientNorms.append(n_tmp)
                    val = ProjectionVal
                    #val = n_tmp*(1-1e-7) # to prevent small numerical errors accumulation
                    values.append(asscalar(val))
                    normed_values.append(asscalar(val/n_tmp))# equals to 0
                    epsilons.append(asscalar((val + dot(prevIterPoint.x, tmp))/n_tmp))
                    
                    if not p.isUC: p.pWarn('addASG is not ajusted with constrained problems handling yet')
                    isConstraint.append(False if p.isUC else True)
                    
                    points.append(prevIterPoint.x)
                    inactive.append(0)
                    nAddedVectors += 1       
                        
#                    else:
#                        p.err('bug in %s, inform openopt developers' % self.__name__)
                        
                indToBeRemovedBySameAngle = []
                
                valDistances1 = asfarray(normed_values)
                valDistances2 = asfarray([(0 if isConstraint[i] else -F0) for i in range(nVec)]) / asfarray(subGradientNorms)
                valDistances3 = asfarray([dot(x-points[i], vec) for i, vec in enumerate(normedSubGradients)])
                
                valDistances = valDistances1 + valDistances2 + valDistances3

                
                #valDistances4 = asfarray([(0 if isConstraint[i] else -F0) for i in range(nVec)]) / asfarray(subGradientNorms)
                
                #valDistancesForExcluding = valDistances1 + valDistances3 + valDistances4 # with constraints it may yield different result vs valDistances
                
#                if p.debug: p.debugmsg('valDistances: ' + str(valDistances))
                if iterInitialDataSize != 0:
                    for j in range(nAddedVectors):
                        ind = -1-j
                        scalarProducts = dot(normedSubGradients, normedSubGradients[ind])
                        IND = where(scalarProducts > 1 - self.sigma)[0]
                        if IND.size != 0:
                            _case = 1
                            if _case == 1:
                                mostUseful = argmax(valDistances[IND])
                                IND = delete(IND, mostUseful)
                                indToBeRemovedBySameAngle +=IND.tolist()
                            else:
                                indToBeRemovedBySameAngle += IND[:-1].tolist()

                indToBeRemovedBySameAngle = list(set(indToBeRemovedBySameAngle)) # TODO: simplify it
                indToBeRemovedBySameAngle.sort(reverse=True)

                if p.debug: p.debugmsg('indToBeRemovedBySameAngle: ' + str(indToBeRemovedBySameAngle) + ' from %d'  %nVec)
                if indToBeRemovedBySameAngle == range(nVec-1, nVec-nAddedVectors-1, -1) and ns > 5:
#                    print 'ns =', ns, 'hs =', hs, 'iterStartPoint.f():', iterStartPoint.f(), 'prevInnerCycleIterStartPoint.f()', prevInnerCycleIterStartPoint.f(), \
#                    'diff:', iterStartPoint.f()-prevInnerCycleIterStartPoint.f()
                    
                    #raise 0
                    p.istop = 17
                    p.msg = 'all new subgradients have been removed due to the angle threshold'
                    return
                                
                #print 'added:', nAddedVectors,'current lenght:', len(values), 'indToBeRemoved:', indToBeRemoved
                
                valDistances = valDistances.tolist()
                valDistances2 = valDistances2.tolist()
                for ind in indToBeRemovedBySameAngle:# TODO: simplify it
                    for List in StoredInfo + [valDistances, valDistances2]:
                        del List[ind]
                nVec -= len(indToBeRemovedBySameAngle)
               
                if nVec > nMaxVec:
                    for List in StoredInfo + [valDistances, valDistances2]:
                        del List[:-nMaxVec]
                    assert len(StoredInfo[-1]) == nMaxVec
                    nVec = nMaxVec
                    
                valDistances = asfarray(valDistances)
                valDistances2 = asfarray(valDistances2)
                
                #F = 0.0

                
                indActive = where(valDistances >= 0)[0]
                #m = len(indActive)
                product = None

                #print('fTol: %f   m: %d   ns: %d' %(fTol, m, ns))
                #raise 0
                if p.debug: p.debugmsg('fTol: %f     ns: %d' %(fTol, ns))
                #Projection = None
                if nVec > 1:
                    normalizedSubGradients = asfarray(normedSubGradients)
                    product = dot(normalizedSubGradients, normalizedSubGradients.T)
                    
                    #best_QP_Point = None
                    
                    #maxQPshoutouts = 1
                    
                    for j in range(maxQPshoutouts if bestFeasiblePoint is not None else 1):
                        F = asscalar(bestFeasiblePoint.f() - fTol * 5**j) if bestFeasiblePoint is not None else nan
                        valDistances2_modified = asfarray([(0 if isConstraint[i] else -F) for i in range(nVec)]) / asfarray(subGradientNorms)
                        ValDistances = valDistances +  valDistances2_modified - valDistances2
                        
                        # DEBUG!!!!!!!!!
                        #ValDistances = array([0, -1])
                        #ValDistances = valDistances
                        # DEBUG END!!!!!!!!!
                
                        # !!!!!!!!!!!!!            TODO: analitical solution for m==2
                        new = 0
                        if nVec == 2 and new:
                            a, b = normedSubGradients[0]*ValDistances[0], normedSubGradients[1]*ValDistances[1]
                            a2, b2, ab = (a**2).sum(), (b**2).sum(), dot(a, b)
                            beta = a2 * (ab-b2) / (ab**2 - a2 * b2)
                            alpha = b2 * (ab-a2) / (ab**2 - a2 * b2)
                            g1 = alpha * a + beta * b
                        else:
                            #projection, koeffs = PolytopProjection(product, asfarray(ValDistances), isProduct = True)   
                            #print 'before PolytopProjection'
                            koeffs = PolytopProjection(product, asfarray(ValDistances), isProduct = True, solver = self.qpsolver)
#                            assert all(isfinite(koeffs))
                            #print koeffs
                            #print 'after PolytopProjection'
                            projection = dot(normalizedSubGradients.T, koeffs).flatten()
                            
                            #print 'norm(projection):', norm(projection)
                            
                            #raise 0
#                            from openopt import QP
#                            p2 = QP(diag(ones(n)), zeros(n), A=-asfarray(normedSubGradients), b=-ValDistances)
#                            projection = p2.solve('cvxopt_qp', iprint=-1).xf
#                            print 'proj:', projection
#                            if itn != 0: raise 0
                            #if ns > 3: raise 0
                            threshold = 1e-9 # for to prevent small numerical issues
                            if j == 0 and any(dot(normalizedSubGradients, projection) < ValDistances * (1-threshold*sign(ValDistances)) - threshold):
                                p.istop = 16
                                p.msg = 'optimal solution wrt required fTol = %g has been obtained' % p.fTol
                                return
                                
                            #p.debugmsg('g1 shift: %f' % norm(g1/norm(g1)-projection/norm(projection)))
                            g1 = projection
                            #if j == 0: 
                                #Projection = projection
                                #ProjectionVal = sum(koeffs*asfarray(ValDistances))
                            #hs = 0.4*norm(g1)
                            M = norm(koeffs, inf)
                            # TODO: remove the cycles
                            indActive = where(koeffs >= M / 1e7)[0]
                            for k in indActive.tolist():
                                inactive[k] = 0
                        NewPoint = Point(x - g1)
                        #print 'isBetter:', NewPoint.betterThan(p.point(x), altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint)

                        if j == 0 or NewPoint.betterThan(best_QP_Point, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint): 
                            best_proj = g1
                            best_QP_Point = NewPoint
                        else:
                            g1 = best_proj
                            break
                            
                    maxQPshoutouts = max((j+2, 1))
                    #print 'opt j:', j, 'nVec:', nVec
                    #Xdist = norm(projection1)
    #                if hs < 0.25*Xdist :
    #                    hs = 0.25*Xdist

                else:
                    g1 = iterStartPoint._getDirection(self.approach, currBestFeasPoint = bestFeasiblePoint)
                    
                if any(isnan(g1)):
                    p.istop = 900
                    return                 
                
                if dilation and len(sh) == 2:
                    point = sh[0] if dot(iterStartPoint._getDirection(self.approach), sh[0]._getDirection(self.approach)) < 0 else sh[1]
                    D.updateDilationMatrix(iterStartPoint._getDirection(self.approach) - point._getDirection(self.approach), alp = 1.2)
                    g1 = D.getDilatedVector(g1)
                    #g1 = tmp
                
                if not isinstance(g1, ndarray) or isinstance(g1, matrix):
                    g1 = g1.A
                
                g1 = g1.flatten()
                
                if any(g1): 
                    g1 /= p.norm(g1)
                else:
                    p.istop = 103 if Point(x).isFeas(False) else -103
                    #raise 0
                    return
                #hs = 1 

                """                           Forward line search                          """

                bestPointBeforeTurn = iterStartPoint
                
                hs_cumsum = 0
                hs_start = hs
                
                hs_mult = 4.0
                for ls in range(p.maxLineSearch):
                    
#                    if ls > 20:
#                        hs_mult = 2.0
#                    elif ls > 10:
#                        hs_mult = 1.5
#                    elif ls > 2:
#                        hs_mult = 1.05
                    
                    assert all(isfinite(g1))
                    assert all(isfinite(x))
                    assert isfinite(hs)
                    x -= hs * g1
                    hs *= hs_mult
                    hs_cumsum += hs

                    newPoint = Point(x) #if ls == 0 else iterStartPoint.linePoint(hs_cumsum/(hs_cumsum-hs), oldPoint) #  TODO: take ls into account?
                    
                    if self.show_nnan: p.info('ls: %d nnan: %d' % (ls, newPoint.__nnan__()))
                    
                    if ls == 0:
                        oldPoint = iterStartPoint#prevIter_best_ls_point#prevIterPoint
                        oldoldPoint = oldPoint
                    assert all(isfinite(oldPoint.x))    
                    #if not self.checkTurnByGradient:
                    
                    #TODO: create routine for modifying bestFeasiblePoint
                    if newPoint.isFeas(False) and (bestFeasiblePoint is None or newPoint.f() > bestFeasiblePoint.f()):
                        bestFeasiblePoint = newPoint
                            
                    if newPoint.betterThan(oldPoint, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint):
                        if newPoint.betterThan(bestPoint, altLinInEq=True): bestPoint = newPoint
                        oldoldPoint = oldPoint
                        #assert dot(oldoldPoint._getDirection(self.approach), g1)>= 0
                        oldPoint, newPoint = newPoint,  None
                    else:
                        bestPointBeforeTurn = oldoldPoint
                        if not itn % 4: 
                            for fn in ['_lin_ineq', '_lin_eq']:
                                if hasattr(newPoint, fn): delattr(newPoint, fn)
                        break

                #assert norm(oldoldPoint.x -newPoint.x) > 1e-17
                hs /= hs_mult
                if ls == p.maxLineSearch-1:
                    p.istop,  p.msg = IS_LINE_SEARCH_FAILED,  'maxLineSearch (' + str(p.maxLineSearch) + ') has been exceeded'
                    return

                p.debugmsg('ls_forward: %d' %ls)
                """                          Backward line search                          """
                #maxLS = 500 #if ls == 0 else 5
                #maxDeltaF = p.ftol / 16.0#fTol/4.0 #p.ftol / 16.0
                #maxDeltaX = p.xtol / 2.0 #if m < 2 else hs / 16.0#Xdist/16.0
                
                #ls_backward = 0
                    
                #DEBUG
#                print '!!!!1:', isPointCovered(oldoldPoint, newPoint, bestFeasiblePoint, fTol), '<<<'
#                print '!!!!2:', isPointCovered(newPoint, oldoldPoint, bestFeasiblePoint, fTol), '<<<'
#                print '!!!!3:', isPointCovered(iterStartPoint, newPoint, bestFeasiblePoint, fTol), '<<<'
#                print '!!!!4:', isPointCovered(newPoint, iterStartPoint, bestFeasiblePoint, fTol), '<<<'
#                raise 0
                #DEBUG END
                
                #assert p.isUC
                maxRecNum = 400#4+int(log2(norm(oldoldPoint.x-newPoint.x)/p.xtol)) 
                #assert dot(oldoldPoint.df(), newPoint.df()) < 0
                #assert sign(dot(oldoldPoint.df(), g1)) != sign(dot(newPoint.df(), g1))
                point1, point2, nLSBackward = LocalizedSearch(oldoldPoint, newPoint, bestFeasiblePoint, fTol, p, maxRecNum, self.approach)
                
                
                #assert sign(dot(point1.df(), g1)) != sign(dot(point2.df(), g1))
                best_ls_point = point1 if point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) else point2

#                if self.doBackwardSearch:
#                    #print '----------------!!!!!!!!  norm(oldoldPoint - newPoint)', norm(oldoldPoint.x -newPoint.x)
#                    isOverHalphPi = True
#                    if isOverHalphPi:
#                        best_ls_point,  bestPointAfterTurn, ls_backward = \
#                        getBestPointAfterTurn(oldoldPoint, newPoint, maxLS = maxLS, maxDeltaF = p.ftol / 2.0, #sf = func, 
#                                            maxDeltaX = p.xtol / 2.0, altLinInEq = True, new_bs = True, checkTurnByGradient = True)
#                        #assert ls_backward != -7
#                    else:
#                        best_ls_point,  bestPointAfterTurn, ls_backward = \
#                        getBestPointAfterTurn(oldoldPoint, newPoint, maxLS = maxLS, maxDeltaF = p.ftol / 2.0, sf = func,  \
#                                            maxDeltaX = p.xtol / 2.0, altLinInEq = True, new_bs = True, checkTurnByGradient = True)       
#
#                    #assert best_ls_point is not iterStartPoint
#                    g1 = bestPointAfterTurn._getDirection(self.approach, currBestFeasPoint = bestFeasiblePoint)
##                    best_ls_point,  bestPointAfterTurn, ls_backward = \
##                    getBestPointAfterTurn(oldoldPoint, newPoint, maxLS = maxLS, maxDeltaF = maxDeltaF, sf = func,  \
##                                          maxDeltaX = maxDeltaX, altLinInEq = True, new_bs = True, checkTurnByGradient = True)
#                p.debugmsg('ls_backward: %d' % ls_backward)
#                if bestPointAfterTurn.betterThan(best_ls_point, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint):
#                    best_ls_point = bestPointAfterTurn
                if oldoldPoint.betterThan(best_ls_point, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint):
                    best_ls_point_with_start = oldoldPoint
                else:
                    best_ls_point_with_start = best_ls_point
                # TODO: extract last point from backward search, that one is better than iterPoint
                if best_ls_point.betterThan(bestPoint, altLinInEq=True): bestPoint = best_ls_point

                if best_ls_point.isFeas(True) and (bestFeasiblePoint is None or best_ls_point.betterThan(bestFeasiblePoint, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint)):
                    bestFeasiblePoint = best_ls_point

    #            print 'ls_backward', ls_backward

    #            if ls_backward < -4:
    #                fTol /= 2.0
    #            elif ls > 4:
    #                fTol *= 2.0
    #                
    #            print 'fTol:', fTol
                
                """                                 Updating hs                                 """
                step_x = p.norm(best_ls_point.x - prevIter_best_ls_point.x)
                #step_f = abs(best_ls_point.f() - prevIter_best_ls_point.f())
                HS.append(hs_start)
                assert ls >= 0
                LS.append(ls)
                p.debugmsg('hs before: %0.1e' % hs)
#                if itn > 3:
#                    mean_ls = (3*LS[-1] + 2*LS[-2]+LS[-3]) / 6.0
#                    j0 = 3.3
#                    #print 'mean_ls:', mean_ls
#                    #print 'ls_backward:', ls_backward
#                    if mean_ls > j0:
#                        hs = (mean_ls - j0 + 1)**0.5 * hs_start
#                    else:
#                        #hs = hs_start / 16.0
#                        if (ls == 0 and ls_backward == -maxLS) or self.maxVectors!=0:
#                            shift_x = step_x / p.xtol
#                            shift_f = step_f / p.ftol
#    #                        print 'shift_x: %e    shift_f: %e' %(shift_x, shift_f)
#                            RD = log10(shift_x+1e-100)
#                            if best_ls_point.isFeas(True) or prevIter_best_ls_point.isFeas(True):
#                                RD = min((RD, log10(shift_f + 1e-100)))
#                            #print 'RD:', RD
#                            if RD > 1.0:
#                                mp = (0.5, (ls/j0) ** 0.5, 1 - 0.2*RD)
#                                hs *= max(mp)

                #prev_hs = hs
                if step_x != 0: 
                    hs = 0.5*step_x                  
#                elif ls  == 0 and nLSBackward > 4:
#                    hs /= 4.0
#                elif ls > 3:
#                    hs *= 2.0
                else:
                    hs = max((hs / 10.0,  p.xtol/2.0))
                
                #if koeffs is not None: hs = sum(koeffs)
                
                p.debugmsg('hs after: %0.1e' % hs)
                    #hs = max((p.xtol/100, 0.5*step_x))
                #print 'step_x:', step_x, 'new_hs:', hs, 'prev_hs:', prev_hs, 'ls:', ls, 'nLSBackward:', nLSBackward

                #if hs < p.xtol/4: hs = p.xtol/4
                
                """                            Handling iterPoints                            """
                   

                if itn == 0:
                    p.debugmsg('hs: ' + str(hs))
                    p.debugmsg('ls: ' + str(ls))
                if self.showLS: p.info('ls: ' + str(ls))
                if self.show_hs: p.info('hs: ' + str(hs))
                if self.show_nnan: p.info('nnan: ' + str(best_ls_point.__nnan__()))
                if self.showRes:
                    r, fname, ind = best_ls_point.mr(True)
                    p.info(fname+str(ind))
                    

                
                #print '^^^^1:>>', iterStartPoint.f(), '2:>>', best_ls_point_with_start.f()
                
                
                #hs = max((norm(best_ls_point_with_start.x-iterStartPoint.x)/2, 64*p.xtol))

                #if p.debug: assert p.isUC

                
                #prevInnerCycleIterStartPoint = iterStartPoint
                
                #if ns > 3: raise 0
                if best_ls_point_with_start.betterThan(iterStartPoint, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint):
                    #raise 0
                    ns = 0
                    iterStartPoint = best_ls_point_with_start
                    break
                else:
                    iterStartPoint = best_ls_point_with_start
                

                

#                if id(best_ls_point_with_start) != id(iterStartPoint): 
#                    print 'new iter point'
#                    assert iterStartPoint.f() != best_ls_point_with_start.f()
#                    if best_ls_point_with_start.betterThan(iterStartPoint, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint):
#                        #hs = norm(best_ls_point_with_start.x-iterStartPoint.x)/16#max(p.xtol, norm(best_ls_point_with_start.x-iterStartPoint.x)/160.0)
#                        ns = 0
#                        
#                        assert not iterStartPoint.betterThan(best_ls_point_with_start)
#                        
#                        iterStartPoint = best_ls_point_with_start
#                        
#                        assert p.isUC
#                        if iterStartPoint.f() - best_ls_point_with_start.f() > fTol :                        
#                            break

#                    else:
#                        raise 0
                # !!!! TODO: has it to be outside the loop?
                
            # "while ns" loop end
            
            isOverHalphPi = product is not None and any(product[indActive].flatten() <= 0)

            if ns == self.maxShoots and isOverHalphPi:
                p.istop = 17
                p.msg = '''
                Max linesearch directions number has been exceeded 
                (probably solution has been obtained), 
                you could increase gsubg parameter "maxShoots" (current value: %d)''' % self.maxShoots
                best_ls_point = best_ls_point_with_start

            """                Some final things for gsubg main cycle                """
            prevIter_best_ls_point = best_ls_point_with_start
            #prevIterPoint = iterStartPoint
            
            
            # TODO: mb move it inside inner loop
            if koeffs is not None:
                indInactive = where(koeffs < M / 1e7)[0]
                for k in indInactive.tolist():
                    inactive[k] += 1
                indInactiveToBeRemoved = where(asarray(inactive) > 5)[0].tolist()                    
#                print ('indInactiveToBeRemoved:'+ str(indInactiveToBeRemoved) + ' from' + str(nVec))
                if p.debug: p.debugmsg('indInactiveToBeRemoved:'+ str(indInactiveToBeRemoved) + ' from' + str(nVec))
                if len(indInactiveToBeRemoved) != 0: # elseware error in current Python 2.6
                    indInactiveToBeRemoved.reverse()# will be sorted in descending order
                    nVec -= len(indInactiveToBeRemoved)
                    for j in indInactiveToBeRemoved:
                        for List in StoredInfo:# + [valDistances.tolist()]:
                            del List[j]     

                
            """                               Call OO iterfcn                                """
            if hasattr(p, '_df'): delattr(p, '_df')
            if best_ls_point.isFeas(False) and hasattr(best_ls_point, '_df'): 
                p._df = best_ls_point.df().copy()           
            assert all(isfinite(best_ls_point.x))
#            print '--------------'
#            print norm(best_ls_point.x-p.xk)
            #if norm(best_ls_point.x-p.xk) == 0: raise 0
            
            cond_same_point = array_equal(best_ls_point.x, p.xk)
            p.iterfcn(best_ls_point)
            #p.iterfcn(bestPointBeforeTurn)

            """                             Check stop criteria                           """

            if cond_same_point and not p.istop:
                #raise 0
                p.istop = 14
                p.msg = 'X[k-1] and X[k] are same'
                p.stopdict[SMALL_DELTA_X] = True
                return
            
            s2 = 0
            if p.istop and not p.userStop:
                if p.istop not in p.stopdict: p.stopdict[p.istop] = True # it's actual for converters, TODO: fix it
                if SMALL_DF in p.stopdict:
                    if best_ls_point.isFeas(False): s2 = p.istop
                    p.stopdict.pop(SMALL_DF)
                if SMALL_DELTA_F in p.stopdict:
                    # TODO: implement it more properly
                    if best_ls_point.isFeas(False) and prevIter_best_ls_point.f() != best_ls_point.f(): s2 = p.istop
                    p.stopdict.pop(SMALL_DELTA_F)
                if SMALL_DELTA_X in p.stopdict:
                    if best_ls_point.isFeas(False) or not prevIter_best_ls_point.isFeas(False) or cond_same_point: s2 = p.istop
                    p.stopdict.pop(SMALL_DELTA_X)
#                if s2 and (any(isnan(best_ls_point.c())) or any(isnan(best_ls_point.h()))) \
#                and not p.isNaNInConstraintsAllowed\
#                and not cond_same_point:
#                    s2 = 0
                    
                if not s2 and any(p.stopdict.values()):
                    for key,  val in p.stopdict.items():
                        if val == True:
                            s2 = key
                            break
                p.istop = s2
                
                for key,  val in p.stopdict.items():
                    if key < 0 or key in set([FVAL_IS_ENOUGH, USER_DEMAND_STOP, BUTTON_ENOUGH_HAS_BEEN_PRESSED]):
                        #p.iterfcn(bestPoint)
                        return
            """                                If stop required                                """
            
            if p.istop:
                    #p.iterfcn(bestPoint)
                    return

isPointCovered2 = lambda pointWithSubGradient, pointToCheck, bestFeasiblePoint, fTol, contol:\
    pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())

def isPointCovered3(pointWithSubGradient, pointToCheck, bestFeasiblePoint, fTol, contol):
    if bestFeasiblePoint is not None \
    and pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()):
        return True
    if not pointWithSubGradient.isFeas(True) and \
    pointWithSubGradient.mr_alt(bestFeasPoint = bestFeasiblePoint) + 1e-15 > \
    dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint)):
        return True
    return False

def isPointCovered4(pointWithSubGradient, pointToCheck, bestFeasiblePoint, fTol, contol):
    #print 'isFeas:', pointWithSubGradient.isFeas(True)
    
#    if pointWithSubGradient.sum_of_all_active_constraints() != 0 and any(pointWithSubGradient.sum_of_all_active_constraints_gradient()):
#        return pointWithSubGradient.sum_of_all_active_constraints() + 0.75*contol > \
#            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.sum_of_all_active_constraints_gradient())
#    
#    return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
    
##############################
    if pointWithSubGradient.isFeas(True):
        # assert bestFeasiblePoint is not None 
        return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
    
    elif pointWithSubGradient.sum_of_all_active_constraints() + 0.75*contol > \
    dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.sum_of_all_active_constraints_gradient()):
        return True
        #return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
    return False
    
#    return pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
#        dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint))
######################
#    
#    if bestFeasiblePoint is not None \
#    and pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()): return True
#    
#    return pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
#            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint))
######################
#    isFeas = pointWithSubGradient.isFeas(True)
#    if isFeas:
#        return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())
#    else:
#        return pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
#            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint))
######################
#    isCoveredByConstraints = pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
#            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint)) \
#            if not pointWithSubGradient.isFeas(True) else True
#    
#    isCoveredByObjective = pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())\
#    if bestFeasiblePoint is not None else True
#    
#    return isCoveredByConstraints and isCoveredByObjective
######################
#    if not pointWithSubGradient.isFeas(True):
#        return True if  pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
#            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint)) else False
#            #, currBestFeasPoint = bestFeasiblePoint)):
#        
#    if pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()):
#        # if pointWithSubGradient is feas (i.e. not 1st case) than bestFeasiblePoint is not None
#        return True
#        
#    return False

isPointCovered = isPointCovered4

def LocalizedSearch(point1, point2, bestFeasiblePoint, fTol, p, maxRecNum, approach):
#    bestFeasiblePoint = None
    contol = p.contol
    for i in range(maxRecNum):
        if p.debug:
            p.debugmsg('req num: %d from %d' % (i, maxRecNum))

        new = 0
        if new:
            if point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) and isPointCovered(point2, point1, bestFeasiblePoint, fTol) \
            or point2.betterThan(point1, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) and isPointCovered(point1, point2, bestFeasiblePoint, fTol):
                break
        else:
            isPoint1Covered = isPointCovered(point2, point1, bestFeasiblePoint, fTol, contol)
            isPoint2Covered = isPointCovered(point1, point2, bestFeasiblePoint, fTol, contol)
            #print 'isPoint1Covered:', isPoint1Covered, 'isPoint2Covered:', isPoint2Covered
            if isPoint1Covered and isPoint2Covered:# and i != 0:
                break
        
        # TODO: prevent small numerical errors accumulation
        point = point1.linePoint(0.5, point2)
        #point = p.point((point1.x + point2.x)/2.0) 
        
        
        if point.isFeas(False) and (bestFeasiblePoint is None or bestFeasiblePoint.f() > point.f()):
            bestFeasiblePoint = point
        
        #if p.debug: assert p.isUC
        if dot(point._getDirection(approach, currBestFeasPoint = bestFeasiblePoint), point1.x-point2.x) < 0:
            point2 = point
        else:
            point1 = point
            
    return point1, point2, i
    
        
######################33
    #                    from scipy.sparse import eye
    #                    from openopt import QP            
    #                    projection2 = QP(eye(p.n, p.n), zeros_like(x), A=polyedr, b = -valDistances).solve('cvxopt_qp', iprint = -1).xf
    #                    g1 = projection2



from numpy import *
import re
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import Vstack, Find, isspmatrix
#import os
try:
    import pyipopt
    pyipoptInstalled = True
except:
    pyipoptInstalled = False
    
class ipopt(baseSolver):
    __name__ = 'ipopt'
    __license__ = "CPL"
    __authors__ = 'Carl Laird (Carnegie Mellon University) and Andreas Wachter'
    __alg__ = "A. Wachter and L. T. Biegler, On the Implementation of a Primal-Dual Interior Point Filter Line Search Algorithm for Large-Scale Nonlinear Programming, Mathematical Programming 106(1), pp. 25-57, 2006 "
    __homepage__ = 'http://www.coin-or.org/'
    __info__ = "requires pyipopt made by Eric Xu You"
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    _canHandleScipySparse = True

    # CHECK ME!
    #__isIterPointAlwaysFeasible__ = lambda self, p: p.__isNoMoreThanBoxBounded__()

    optFile = 'auto'
    options = ''

    def __init__(self): pass
    def __solver__(self, p):
        if not pyipoptInstalled:
            p.err('you should have pyipopt installed')
            
#        try:
#            os.close(1); os.close(2) # may not work for non-Unix OS
#        except:
#            pass
            
        nvar = p.n
        x_L = p.lb
        x_U = p.ub

        ncon = p.nc + p.nh + p.b.size + p.beq.size

        g_L, g_U = zeros(ncon), zeros(ncon)
        g_L[:p.nc] = -inf
        g_L[p.nc+p.nh:p.nc+p.nh+p.b.size] = -inf

        
        # IPOPT non-linear constraints, both eq and ineq
        if p.isFDmodel:
            r = []
            if p.nc != 0: r.append(p._getPattern(p.user.c))
            if p.nh != 0: r.append(p._getPattern(p.user.h))
            if p.nb != 0: r.append(p.A)
            if p.nbeq != 0: r.append(p.Aeq)
            if len(r)>0:
                if all([isinstance(elem, ndarray) for elem in r]):
                    r = vstack(r)
                else:
                    r = Vstack(r)
                    if isspmatrix(r):
                        from scipy import __version__
                        if __version__.startswith('0.7.3') or __version__.startswith('0.7.2') or __version__.startswith('0.7.1') or __version__.startswith('0.7.0'):
                            p.pWarn('updating scipy to version >= 0.7.4 is very recommended for the problem with the solver IPOPT')
            else:
                r = array([])
            
            if isspmatrix(r):
                I, J, _ = Find(r)
                # DON'T remove it!
                I, J = array(I, int64), array(J, int64)
            
            elif isinstance(r, ndarray):
                if r.size == 0:
                    I, J= array([], dtype=int64),array([], dtype=int64)
                else:
                    I, J = where(r)
            
            else:
                p.disp('unimplemented type:%s' % str(type(r))) # dense matrix? 
                
            
            nnzj = len(I)
        else:
            I, J = where(ones((ncon, p.n)))
            #I, J = None, None
            nnzj = ncon * p.n #TODO: reduce it
        

        def eval_g(x):
            r = array(())
            if p.userProvided.c: r = p.c(x)
            if p.userProvided.h: r = hstack((r, p.h(x)))
            r = hstack((r, p._get_AX_Less_B_residuals(x), p._get_AeqX_eq_Beq_residuals(x)))
            return r

        def eval_jac_g(x, flag, userdata = (I, J)):
            (I, J) = userdata
            if  flag and p.isFDmodel: 
                return (I, J) 
            r = []
            if p.userProvided.c: r.append(p.dc(x))
            if p.userProvided.h: r.append(p.dh(x))
            if p.nb != 0: r.append(p.A)
            if p.nbeq != 0: r.append(p.Aeq)
            # TODO: fix it!
            if any([isspmatrix(elem) for elem in r]):
                r = Vstack([(atleast_2d(elem) if elem.ndim < 2 else elem) for elem in r])
            elif len(r)!=0:
                r = vstack(r)
            
            if p.isFDmodel: 
                # TODO: make it more properly
                R = (r.tocsr() if isspmatrix(r) else r)[I, J]
                if isspmatrix(R): 
                    return R.A
                elif isinstance(R, ndarray): 
                    return R
                else: p.err('bug in OpenOpt-ipopt connection, inform OpenOpt developers, type(R) = %s' % type(R))
            if flag:
                return (I, J)
            else:
                if isspmatrix(r): r = r.A
                return r.flatten()


        """ This function might be buggy, """ # // comment by Eric
        nnzh = 0
        def eval_h(lagrange, obj_factor, flag):
            return None



#        def apply_new(x):
#            return True

        nlp = pyipopt.create(nvar, x_L, x_U, ncon, g_L, g_U, nnzj, nnzh, p.f, p.df, eval_g, eval_jac_g)

        if self.optFile == 'auto':
            lines = ['# generated automatically by OpenOpt\n','print_level 0\n']
            lines.append('tol ' + str(p.ftol)+ '\n')
            lines.append('constr_viol_tol ' + str(p.contol)+ '\n')
            lines.append('max_iter ' + str(min(15000, p.maxIter))+ '\n')
            if self.options != '' :
                for s in re.split(',|;', self.options):
                    lines.append(s.strip().replace('=', ' ',  1) + '\n')
            if p.nc == 0:
                lines.append('jac_d_constant yes\n')
            if p.nh == 0:
                lines.append('jac_c_constant yes\n')
            if p.castFrom.lower() in ('lp', 'qp', 'llsp'):
                lines.append('hessian_constant yes\n')


            ipopt_opt_file = open('ipopt.opt', 'w')
            ipopt_opt_file.writelines(lines)
            ipopt_opt_file.close()


        try:
            x, zl, zu, obj  = nlp.solve(p.x0)[:4]
            if p.point(p.xk).betterThan(p.point(x)):
                obj = p.fk
                p.xk = p.xk.copy() # for more safety
            else:
                p.xk, p.fk = x.copy(), obj
            if p.istop == 0: p.istop = 1000
        finally:
            nlp.close()





from baseProblem import NonLinProblem
from NLP import NLP
from numpy import max, array, hstack, vstack, zeros, ones, inf, asfarray, atleast_2d

class MMP(NonLinProblem):
    """
    Mini-Max Problem
    """
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    expectedArgs = ['f', 'x0']
    allowedGoals = ['minimax']
    probType = 'MMP'
    isObjFunValueASingleNumber = False
    showGoal = True
    goal = 'minimax'
    
    def __init__(self, *args, **kwargs):
        NonLinProblem.__init__(self, *args, **kwargs)

    def objFuncMultiple2Single(self, fv):
        return max(fv)

    def mmp2nlp(self, solver, **solver_params):
        f = lambda x: x[-1]
        DF = array([0]*self.n + [1])
        df = lambda x: DF.copy()

        def iterfcn(*args,  **kwargs):
            p2.primalIterFcn(*args,  **kwargs)

            self.xk = p2.xk[:-1].copy()
            self.fk = p2.fk
            self.rk = p2.rk

            self.istop = p2.istop

            if self.istop and p2.rk <= p2.contol:
                self.msg = p2.msg
            self.iterfcn()

        p2 = NLP(f, hstack((self.x0, max(self.f(self.x0)))), df=df, xtol = self.xtol, ftol = self.ftol, gtol = self.gtol,\
        A=hstack((self.A, zeros((len(self.b), 1)))),  b=self.b,  Aeq=hstack((self.Aeq, zeros((len(self.beq), 1)))),  beq=self.beq,  lb=hstack((self.lb, -inf)),  ub=hstack((self.ub, inf)), \
        maxFunEvals = self.maxFunEvals, fEnough = self.fEnough, maxIter=self.maxIter, iprint = -1, \
        maxtime = self.maxTime, maxCPUTime = self.maxCPUTime,  noise = self.noise)

        if self.userProvided.c:
            arr_dc = array([0]*self.nc + [-1]*self.nf).reshape(-1, 1)
            p2.c = lambda x: hstack((self.c(x[:-1]), self.f(x[:-1])-x[-1]))
            p2.dc = lambda x: hstack((vstack((self.dc(x[:-1]), self.df(x[:-1]))), arr_dc))
        else:
            p2.c = lambda x: self.f(x[:-1])-x[-1]
            arr_dc = -ones((self.nf, 1))
            p2.dc = lambda x: hstack((atleast_2d(self.df(x[:-1]),  arr_dc)))
        if self.userProvided.h:
            arr_dh = array([0]*self.nh).reshape(-1, 1)
            p2.h = lambda x: self.h(x[:-1])
            p2.dh = lambda x: hstack((atleast_2d(self.dh(x[:-1])), arr_dh))


        p2.primalIterFcn,  p2.iterfcn = p2.iterfcn, iterfcn

        #p2.checkdc()

        r2 = p2.solve(solver)
        #xf = fsolve(self.f, self.x0, fprime=self.df, xtol = self.xtol, maxfev = self.maxFunEvals)
        xf = r2.xf[:-1]
        self.xk = self.xf = xf
        self.fk = self.ff = max(self.f(xf))




"""
Example of using additional parameters for user f, c, h functions
Note! For oofun handling user parameters is performed 
in the same way: 
my_oofun.args = (...)
they will be passed to derivative function as well (if you have supplied it)
"""

from openopt import NLP
from numpy import asfarray

f = lambda x, a: (x**2).sum() + a * x[0]**4
x0 = [8, 15, 80]
p = NLP(f, x0)


#using c(x)<=0 constraints
p.c = lambda x, b, c: (x[0]-4)**2 - 1 + b*x[1]**4 + c*x[2]**4

#using h(x)=0 constraints
p.h = lambda x, d: (x[2]-4)**2 + d*x[2]**4 - 15
    
p.args.f = 4 # i.e. here we use a=4
# so it's the same to "a = 4; p.args.f = a" or just "p.args.f = a = 4" 

p.args.c = (1,2)

p.args.h = 15 

# Note 1: using tuple p.args.h = (15,) is valid as well

# Note 2: if all your funcs use same args, you can just use 
# p.args = (your args)

# Note 3: you could use f = lambda x, a: (...); c = lambda x, a, b: (...); h = lambda x, a: (...)

# Note 4: if you use df or d2f, they should handle same additional arguments;
# same to c - dc - d2c, h - dh - d2h

# Note 5: instead of myfun = lambda x, a, b: ...
# you can use def myfun(x, a, b): ...

r = p.solve('ralg')
"""
If you will encounter any problems with additional args implementation, 
you can use the simple python trick
p.f = lambda x: other_f(x, <your_args>)
same to c, h, df, etc
"""



PythonMax = max
from numpy import all, any, isfinite, inf, nan, ndarray, ones, \
ravel, asfarray, prod, asarray, atleast_1d, isinf
import numpy as np
from oologfcn import *
from oographics import Graphics
from setDefaultIterFuncs import setDefaultIterFuncs, denyingStopFuncs
from nonLinFuncs import nonLinFuncs
from residuals import residuals
from ooIter import ooIter

#from Point import Point currently lead to bug
from openopt.kernel.Point import Point

from iterPrint import ooTextOutput
from ooMisc import setNonLinFuncsNumber, assignScript, norm
from nonOptMisc import isspmatrix, scipyInstalled, scipyAbsentMsg, csr_matrix
from copy import copy as Copy
try:
    from DerApproximator import check_d1
    DerApproximatorIsInstalled = True
except:
    DerApproximatorIsInstalled = False

ProbDefaults = {'diffInt': 1.5e-8,  'xtol': 1e-6,  'noise': 0}
from runProbSolver import runProbSolver
import GUI

from fdPrepare import fdPrepare


class user:
    def __init__(self):
        pass

class oomatrix:
    def __init__(self):
        pass
    def matMultVec(self, x, y):
        return np.dot(x, y) if not isspmatrix(x) else x._mul_sparse_matrix(csr_matrix(y.reshape((y.size, 1)))).A.flatten() 
    def matmult(self, x, y):
        return np.dot(x, y)
        #return asarray(x) ** asarray(y)
    def dotmult(self, x, y):
        return x * y
        #return asarray(x) * asarray(y)

class autocreate:
    def __init__(self): pass

class baseProblem(oomatrix, residuals, ooTextOutput):
    isObjFunValueASingleNumber = True
    manage = GUI.manage # GUI func
    #_useGUIManager = False # TODO: implement it
    prepared = False
    _baseProblemIsPrepared = False
    isManagerUsed = False
    managerTextOutput = True
    
    name = 'unnamed'
    state = 'init'# other: paused, running etc
    castFrom = '' # used by converters qp2nlp etc
    nonStopMsg = ''
    xlabel = 'time'
    plot = False # draw picture or not
    show = True # use command pylab.show() after solver finish or not
    _immutable  = False
    

    iter = 0
    cpuTimeElapsed = 0.
    TimeElapsed = 0.
    isFinished = False
    invertObjFunc = False # True for goal = 'max' or 'maximum'
    nProc = 1 # number of processors to use

    lastPrintedIter = -1
    
    iterObjFunTextFormat = '%0.3e'
    finalObjFunTextFormat = '%0.8g'
    debug = 0
    
    iprint = 10
    #if iprint<0 -- no output
    #if iprint==0 -- final output only

    maxDistributionSize = 0 # used in stochastic problems

    maxIter = 1000
    maxFunEvals = 10000 # TODO: move it to NinLinProblem class?
    maxCPUTime = inf
    maxTime = inf
    maxLineSearch = 500 # TODO: move it to NinLinProblem class?
    xtol = ProbDefaults['xtol'] # TODO: move it to NinLinProblem class?
    gtol = 1e-6 # TODO: move it to NinLinProblem class?
    ftol = 1e-6
    contol = 1e-6
    discrtol = 1e-5 # tolerance required for discrete constraints 
    fTol = None
    rTol = 1e-8#used in interalg
    f_bound_distance = nan
    f_bound_estimation = nan

    minIter = 0
    minFunEvals = 0
    minCPUTime = 0.0
    minTime = 0.0
    
    storeIterPoints = False 

    userStop = False # becomes True is stopped by user
    
    useSparse = 'auto' # involve sparse matrices: 'auto' (autoselect, premature) | True | False
    useAttachedConstraints = False

    x0 = None
    isFDmodel = False # OO kernel set it to True if oovars/oofuns are used

    noise = ProbDefaults['noise'] # TODO: move it to NinLinProblem class?

    showFeas = False
    useScaledResidualOutput = False

    hasLogicalConstraints = False
    
    _isStochastic = False
    
    # A * x <= b inequalities
    A = None
    b = None

    # Aeq * x = b equalities
    Aeq = None
    beq = None
    
    #non-linear constraints
    c = None # c(x)<=0
    h = None # h(x)=0

    
    scale = None

    goal = None# should be redefined by child class
    # possible values: 'maximum', 'min', 'max', 'minimum', 'minimax' etc
    showGoal = False# can be redefined by child class, used for text & graphic output

    color = 'b' # blue, color for plotting
    specifier = '-'# simple line for plotting
    plotOnlyCurrentMinimum = False # some classes like GLP change the default to True
    xlim = (nan,  nan)
    ylim = (nan,  nan)
    legend = ''

    fixedVars = None
    freeVars = None
    
    istop = 0
    
    maxSolutions = 1 # used in interalg and mb other solvers

    fEnough = -inf # if value less than fEnough will be obtained
    # and all constraints no greater than contol
    # then solver will be stopped.
    # this param is handled in iterfcn of OpenOpt kernel
    # so it may be ignored with some solvers not closely connected to OO kernel

    fOpt = None # optimal value, if known
    implicitBounds = inf
    
    convex = 'unknown' # used in interalg
    _linear_objective = False # used in interalg
    _textFuncsInitialized = False


    def __init__(self, *args, **kwargs):
        # TODO: add the field to ALL classes
        self._setTextFuncs()
        self.data4TextOutput = ['objFunVal', 'log10(maxResidual)']
        self.nEvals = {}
        self.finish = []# functions f_i(r,p) to be evaluated after finish
        self._bg_color = '#FFFFF0'
        
        if hasattr(self, 'expectedArgs'): 
            if len(self.expectedArgs)<len(args):
                self.err('Too much arguments for '+self.probType +': '+ str(len(args)) +' are got, at most '+ str(len(self.expectedArgs)) + ' were expected')
            for i, arg in enumerate(args):
                setattr(self, self.expectedArgs[i], arg)
        self.norm = norm
        self.denyingStopFuncs = denyingStopFuncs()
        self.iterfcn = lambda *args, **kwargs: ooIter(self, *args, **kwargs)# this parameter is only for OpenOpt developers, not common users
        self.graphics = Graphics()
        self.user = user()
        self.F = lambda x: self.objFuncMultiple2Single(self.objFunc(x)) # TODO: should be changes for LP, MILP, QP classes!

        self.point = lambda *args,  **kwargs: Point(self, *args,  **kwargs)

        self.timeElapsedForPlotting = [0.]
        self.cpuTimeElapsedForPlotting = [0.]
        #user can redirect these ones, as well as debugmsg
        self.debugmsg = lambda msg: oodebugmsg(self,  msg)
        
        self.constraints = [] # used in isFDmodel

        self.callback = [] # user-defined callback function(s)
        
        self.solverParams = autocreate()

        self.userProvided = autocreate()

        self.special = autocreate()

        self.intVars = [] # for problems like MILP
        self.binVars = [] # for problems like MILP
        self.optionalData = []#string names of optional data like 'c', 'h', 'Aeq' etc
        
        if self.allowedGoals is not None: # None in EIG
            if 'min' in self.allowedGoals:
                self.minimize = lambda *args, **kwargs: minimize(self, *args, **kwargs)
            if 'max' in self.allowedGoals:
                self.maximize = lambda *args, **kwargs: maximize(self, *args, **kwargs)
                
        assignScript(self, kwargs)
    
 
    def __setattr__(self, attr, val): 
#        print attr
        if self._immutable and attr != '_immutable': 
            self.err('''
            The OpenOpt problem instance is immutable here, 
            arguments should pass to constructor or solve()'''
            )
        self.__dict__[attr] = val
    
    def _setTextFuncs(self):   
        if self._textFuncsInitialized: 
            return
        self.err =  ooerr
        self.warn = oowarn
        self.info = ooinfo
        self.hint = oohint
        self.pWarn = ooPWarn
        self.disp = oodisp
        self._textFuncsInitialized = True

    def __finalize__(self):
        if self.isFDmodel:
            self.xf = self._vector2point(self.xf)
            self.constraints = self._initial_constraints

    def objFunc(self, x):
        return self.f(x) # is overdetermined in LP, QP, LLSP etc classes

    def __isFiniteBoxBounded__(self): # TODO: make this function 'lazy'
        return all(isfinite(self.ub)) and all(isfinite(self.lb))

    def __isNoMoreThanBoxBounded__(self): # TODO: make this function 'lazy'
        return self.b.size ==0 and self.beq.size==0 and (self._baseClassName == 'Matrix' or (not self.userProvided.c and not self.userProvided.h))

#    def __1stBetterThan2nd__(self,  f1, f2,  r1=None,  r2=None):
#        if self.isUC:
#            #TODO: check for goal = max/maximum
#            return f1 < f2
#        else:#then r1, r2 should be defined
#            return (r1 < r2 and  self.contol < r2) or (((r1 <= self.contol and r2 <=  self.contol) or r1==r2) and f1 < f2)
#
#    def __1stCertainlyBetterThan2ndTakingIntoAcoountNoise__(self,   f1, f2,  r1=None,  r2=None):
#        if self.isUC:
#            #TODO: check for goalType = max
#            return f1 + self.noise < f2 - self.noise
#        else:
#            #return (r1 + self.noise < r2 - self.noise and  self.contol < r2) or \
#            return (r1 < r2  and  self.contol < r2) or \
#            (((r1 <= self.contol and r2 <=  self.contol) or r1==r2) and f1 + self.noise < f2 - self.noise)


    def solve(self, *args, **kwargs):
        return runProbSolver(self, *args, **kwargs)
        
    def _solve(self, *args, **kwargs):
        self.debug = True
        return self.solve(*args, **kwargs)
    
    def objFuncMultiple2Single(self, f):
        #this function can be overdetermined by child class
        if asfarray(f).size != 1: self.err('unexpected f size. The function should be redefined in OO child class, inform OO developers')
        return f

    def fill(self, newProb, sameConstraints=True):
        # fills some fields of new prob with old prob values
        newProb.castFrom = self.probType

        #TODO: hold it in single place

        fieldsToAssert = ['contol', 'xtol', 'ftol', 'gtol', 'iprint', 'maxIter', 'maxTime', 'maxCPUTime','fEnough', 'goal', 'color', 'debug', 'maxFunEvals', 'xlabel']
        # TODO: boolVars, intVars
        if sameConstraints: fieldsToAssert+= ['lb', 'ub', 'A', 'Aeq', 'b', 'beq']

        for key in fieldsToAssert:
            if hasattr(self, key): setattr(newProb, key, getattr(self, key))


        # note: because of 'userProvided' from prev line
        #self self.userProvided is same to newProb.userProvided
        
#        for key in ['f','df', 'd2f']:
#                if hasattr(self.userProvided, key) and getattr(self.userProvided, key):
#                    setattr(newProb, key, getattr(self.user, key))
        
        Arr = ['f', 'df']
        if sameConstraints:
            Arr += ['c','dc','h','dh','d2c','d2h']
        
        for key in Arr:
            if hasattr(self.userProvided, key):
                if getattr(self.userProvided, key):
                    #setattr(newProb, key, getattr(self.user, key))
                    setattr(newProb, key, getattr(self, key)) if self.isFDmodel else setattr(newProb, key, getattr(self.user, key))
                else:
                    setattr(newProb, key, None)
                        
    FuncDesignerSign = 'f'

    def _isFDmodel(self):
        try:
            #from FuncDesigner.ooFun import oofun
            from FuncDesigner import ooarray, oofun
        except ImportError:
            return False
        fds = getattr(self, self.FuncDesignerSign, None)
        if fds is None:
            return False        
        if isinstance(fds, (oofun, ooarray)):
            return True
        if isinstance(fds, dict):
            return True if isinstance(list(fds.keys())[0], (oofun, ooarray)) else False
        if isinstance(fds, (list, tuple, ndarray)):
            if isinstance(fds[0], (oofun, ooarray)):
                return True
            elif isinstance(fds[0], (list, tuple, ndarray)):
                return isinstance(fds[0][0], (oofun, ooarray))
        return False
    
    # Base class method
    def _prepare(self): 
        if self._baseProblemIsPrepared: return
        if self.useSparse == 0:
            self.useSparse = False
        elif self.useSparse == 1:
            self.useSparse = True
        if self.useSparse == 'auto' and not scipyInstalled:
            self.useSparse = False
        if self.useSparse == True and not scipyInstalled:
            self.err("You can't set useSparse=True without scipy installed")
        if self._isFDmodel():
            fdPrepare(self)
        else: # not FuncDesigner
            if self.fixedVars is not None or self.freeVars is not None:
                self.err('fixedVars and freeVars are valid for optimization of FuncDesigner models only')
        if self.x0 is None: 
            arr = ['lb', 'ub']
            if self.probType in ['LP', 'MILP', 'QP', 'SOCP', 'SDP']: arr.append('f')
            if self.probType in ['LLSP', 'LLAVP', 'LUNP']: arr.append('D')
            for fn in arr:
                if not hasattr(self, fn): continue
                tmp = getattr(self, fn)
                fv = asarray(tmp) if not isspmatrix(tmp) else tmp.A
                if any(isfinite(fv)):
                    self.x0 = np.zeros(fv.size)
                    break
        self.x0 = ravel(self.x0)
        
        if not hasattr(self, 'n'): self.n = self.x0.size
        if not hasattr(self, 'lb'): self.lb = -inf * ones(self.n)
        if not hasattr(self, 'ub'): self.ub =  inf * ones(self.n)        
        self._lb, self._ub = np.copy(self.lb), np.copy(self.ub)

        for fn in ('A', 'Aeq'):
            fv = getattr(self, fn)
            if fv is not None:
                #afv = asfarray(fv) if not isspmatrix(fv) else fv.toarray() # TODO: omit casting to dense matrix
                afv = asfarray(fv)  if type(fv) in [list, tuple] else fv
                if len(afv.shape) > 1:
                    if afv.shape[1] != self.n:
                        self.err('incorrect ' + fn + ' size')
                else:
                    if afv.shape != () and afv.shape[0] == self.n: afv = afv.reshape(1, self.n)
                setattr(self, fn, afv)
            else:
                setattr(self, fn, asfarray([]).reshape(0, self.n))
                
        nA, nAeq = prod(self.A.shape), prod(self.Aeq.shape) 
        SizeThreshold = 2 ** 15
        if scipyInstalled:
            from scipy.sparse import csc_matrix
            if isspmatrix(self.A) or (nA > SizeThreshold  and np.flatnonzero(self.A).size < 0.25*nA):
                self._A = csc_matrix(self.A)
            if isspmatrix(self.Aeq) or (nAeq > SizeThreshold and np.flatnonzero(self.Aeq).size < 0.25*nAeq):
                self._Aeq = csc_matrix(self.Aeq)
            
        elif nA > SizeThreshold or nAeq > SizeThreshold:
            self.pWarn(scipyAbsentMsg)
            
        self._baseProblemIsPrepared = True


class MatrixProblem(baseProblem):
    _baseClassName = 'Matrix'
    ftol = 1e-8
    contol = 1e-8
    #obsolete, should be removed
    # still it is used by lpSolve
    # Awhole * x {<= | = | >= } b
    Awhole = None # matrix m x n, n = len(x)
    bwhole = None # vector, size = m x 1
    dwhole = None #vector of descriptors, size = m x 1
    # descriptors dwhole[j] should be :
    # 1 : <Awhole, x> [j] greater (or equal) than bwhole[j]
    # -1 : <Awhole, x> [j] less (or equal) than bwhole[j]
    # 0 : <Awhole, x> [j] = bwhole[j]
    def __init__(self, *args, **kwargs):
        baseProblem.__init__(self, *args, **kwargs)
        self.kernelIterFuncs = setDefaultIterFuncs('Matrix')
        self.userProvided.c = False
        self.userProvided.h = False

    def _Prepare(self):
        if self.prepared == True:
            return
        baseProblem._prepare(self)
        self.prepared = True

    # TODO: move the function to child classes
    def _isUnconstrained(self):
        if  self.b.size !=0 or self.beq.size != 0: 
            return False
        
        # for PyPy compatibility
        if any(atleast_1d(self.lb) != -inf) or any(atleast_1d(self.ub) != inf):
            return False
            
        return True


class Parallel:
    def __init__(self):
        self.f = False# 0 - don't use parallel calclations, 1 - use
        self.c = False
        self.h = False
        #TODO: add paralell func!
        #self.parallel.fun = dfeval

class Args:
    def __init__(self): pass
    f, c, h = (), (), ()

class NonLinProblem(baseProblem, nonLinFuncs, Args):
    _baseClassName = 'NonLin'
    diffInt = ProbDefaults['diffInt']        #finite-difference gradient aproximation step
    #lines with |info_user-info_numerical| / (|info_user|+|info_numerical+1e-15) greater than maxViolation will be shown
    maxViolation = 1e-2
    JacobianApproximationStencil = 1
    def __init__(self, *args, **kwargs):
        baseProblem.__init__(self, *args, **kwargs)
        if not hasattr(self, 'args'): self.args = Args()
        self.prevVal = {}
        for fn in ['f', 'c', 'h', 'df', 'dc', 'dh', 'd2f', 'd2c', 'd2h']:
            self.prevVal[fn] = {'key':None, 'val':None}

        self.functype = {}

        #self.isVectoriezed = False

#        self.fPattern = None
#        self.cPattern = None
#        self.hPattern = None
        self.kernelIterFuncs = setDefaultIterFuncs('NonLin')

    def checkdf(self, *args,  **kwargs):
        return self.checkGradient('df', *args,  **kwargs)

    def checkdc(self, *args,  **kwargs):
        return self.checkGradient('dc', *args,  **kwargs)

    def checkdh(self, *args,  **kwargs):
        return self.checkGradient('dh', *args,  **kwargs)
    
    def checkGradient(self, funcType, *args,  **kwargs):
        self._Prepare()
        if not DerApproximatorIsInstalled:
            self.err('To perform gradients check you should have DerApproximator installed, see http://openopt.org/DerApproximator')
        
        if not getattr(self.userProvided, funcType):
            self.warn("you haven't analitical gradient provided for " + funcType[1:] + ', turning derivatives check for it off...')
            return
        if len(args)>0:
            if len(args)>1 or 'x' in kwargs:
                self.err('checkd<func> funcs can have single argument x only (then x should be absent in kwargs )')
            xCheck = asfarray(args[0])
        elif 'x' in kwargs:
            xCheck = asfarray(kwargs['x'])
        else:
            xCheck = asfarray(self.x0)
        
        maxViolation = 0.01
        if 'maxViolation' in kwargs:
            maxViolation = kwargs['maxViolation']
            
        self.disp(funcType + (': checking user-supplied gradient of shape (%d, %d)' % (getattr(self, funcType[1:])(xCheck).size, xCheck.size)))
        self.disp('according to:')
        self.disp('    diffInt = ' + str(self.diffInt)) # TODO: ADD other parameters: allowed epsilon, maxDiffLines etc
        self.disp('    |1 - info_user/info_numerical| < maxViolation = '+ str(maxViolation))        
        
        check_d1(getattr(self, funcType[1:]), getattr(self, funcType), xCheck, **kwargs)
        
        # reset counters that were modified during check derivatives
        self.nEvals[funcType[1:]] = 0
        self.nEvals[funcType] = 0
        
    def _makeCorrectArgs(self):
        argslist = dir(self.args)
        if not ('f' in argslist and 'c' in argslist and 'h' in argslist):
            tmp, self.args = self.args, autocreate()
            self.args.f = self.args.c = self.args.h = tmp
        for j in ('f', 'c', 'h'):
            v = getattr(self.args, j)
            if type(v) != type(()): setattr(self.args, j, (v,))

#    def __finalize__(self):
#        BaseProblem.__finalize__(self)
##        if self.isFDmodel:
##            self.xf = self._vector2point(self.xf)

    def _Prepare(self):
        baseProblem._prepare(self)
        if asarray(self.implicitBounds).size == 1:
            self.implicitBounds = [-self.implicitBounds, self.implicitBounds]
            self.implicitBounds.sort()# for more safety, maybe user-provided value is negative
        if hasattr(self, 'solver'):
            if not self.solver.iterfcnConnected:
                if self.solver.funcForIterFcnConnection == 'f':
                    if not hasattr(self, 'f_iter'):
                        self.f_iter = PythonMax(self.n, 4)
                else:
                    if not hasattr(self, 'df_iter'):
                        self.df_iter = True
        
        if self.prepared == True:
            return
        
        # TODO: simplify it
        self._makeCorrectArgs()
        for s in ('f', 'df', 'd2f', 'c', 'dc', 'd2c', 'h', 'dh', 'd2h'):
            derivativeOrder = len(s)-1
            self.nEvals[Copy(s)] = 0
            Attr = getattr(self, s, None)
            if Attr is not None and (not isinstance(Attr, (list, tuple)) or len(Attr) != 0) :
                setattr(self.userProvided, s, True)

                A = getattr(self,s)

                if type(A) not in [list, tuple]: #TODO: add or ndarray(A)
                    A = (A,)#make tuple
                setattr(self.user, s, A)
            else:
                setattr(self.userProvided, s, False)
            if derivativeOrder == 0:
                setattr(self, s, lambda x, IND=None, userFunctionType= s, ignorePrev=False, getDerivative=False: \
                        self.wrapped_func(x, IND, userFunctionType, ignorePrev, getDerivative))
                
#                setattr(self, s, lambda x, IND=None, userFunctionType= s, ignorePrev=False, getDerivative=False, \
#                        _linePointDescriptor = None: \
#                        self.wrapped_func(x, IND, userFunctionType, ignorePrev, getDerivative, _linePointDescriptor))
            elif derivativeOrder == 1:
                setattr(self, s, lambda x, ind=None, funcType=s[-1], ignorePrev = False, useSparse=self.useSparse:
                        self.wrapped_1st_derivatives(x, ind, funcType, ignorePrev, useSparse))
            elif derivativeOrder == 2:
                setattr(self, s, getattr(self, 'user_'+s))
            else:
                self.err('incorrect non-linear function case')

        self.diffInt = ravel(self.diffInt)
        
        # TODO: mb get rid of the field
        self.vectorDiffInt = self.diffInt.size > 1
        
        if self.scale is not None:
            self.scale = ravel(self.scale)
            if self.vectorDiffInt or self.diffInt[0] != ProbDefaults['diffInt']:
                self.info('using both non-default scale & diffInt is not recommended. diffInt = diffInt/scale will be used')
            self.diffInt = self.diffInt / self.scale
       

        #initialization, getting nf, nc, nh etc:
        for s in ['c', 'h', 'f']:
            if not getattr(self.userProvided, s):
                setattr(self, 'n'+s, 0)
            else:
                setNonLinFuncsNumber(self,  s)
                
        self.prepared = True

    # TODO: move the function to derived classes
    _isUnconstrained = lambda self:\
        self.b.size ==0 and self.beq.size==0 and not self.userProvided.c and not self.userProvided.h \
        and (len(self.lb)==0 or all(isinf(self.lb))) and (len(self.ub)==0 or all(isinf(self.ub)))
    

def minimize(p, *args, **kwargs):
    if 'goal' in kwargs:
        if kwargs['goal'] in ['min', 'minimum']:
            p.warn("you shouldn't pass 'goal' to the function 'minimize'")
        else:
            p.err('ambiguous goal has been requested: function "minimize", goal: %s' %  kwargs['goal'])
    p.goal = 'minimum'
    return runProbSolver(p, *args, **kwargs)

def maximize(p, *args, **kwargs):
    if 'goal' in kwargs:
        if kwargs['goal'] in ['max', 'maximum']:
            p.warn("you shouldn't pass 'goal' to the function 'maximize'")
        else:
            p.err('ambiguous goal has been requested: function "maximize", goal: %s' %  kwargs['goal'])
    p.goal = 'maximum'
    return runProbSolver(p, *args, **kwargs)            



from numpy.linalg import norm
from numpy import dot, asfarray, atleast_1d,  zeros, ones, int, float64, where, inf, linalg, ndarray, prod
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import scipyAbsentMsg, isspmatrix

try:
    import scipy
    scipyInstalled = True
    from scipy.sparse import linalg
except:
    scipyInstalled = False
    


class defaultSLEsolver(baseSolver):
    __name__ = 'defaultSLEsolver'
    __license__ = "BSD"
    __authors__ = 'Dmitrey'
    __alg__ = ''
    __info__ = ''
    matrixSLEsolver = 'autoselect'
    sparseSolvers = ['bicg', 'bicgstab', 'cg', 'cgs', 'gmres', 'minres', 'qmr', 'spsolve']
    denseSolvers = ['numpy_linalg_solve']
    defaultSparseSolver = 'spsolve'
    defaultDenseSolver = 'numpy_linalg_solve'
    #__optionalDataThatCanBeHandled__ = []

    def __init__(self): pass

    def __solver__(self, p):
        #p.debug = 1
        # TODO: code clean up
        solver = self.matrixSLEsolver
        if solver in self.denseSolvers:
            useDense = True
        elif solver in self.sparseSolvers:
            useDense = False
        elif solver == 'autoselect':
            if not scipyInstalled:
                useDense = True
                if isspmatrix(p.C) and prod(p.C.shape)>100000:
                        s = """You have no scipy installed .
                        Thus the SLE will be solved as dense. 
                        """
                        p.pWarn(s)
                    
                solver = self.defaultDenseSolver
                self.matrixSLEsolver = solver
                
            else:
                solver = self.defaultSparseSolver
                self.matrixSLEsolver = solver
                useDense = False
        else:
            p.err('Incorrect SLE solver (%s)' % solver)
                
        if isinstance(solver, str): 
            if solver == 'numpy_linalg_solve':
                solver = linalg.solve
            else:
                solver = getattr(scipy.sparse.linalg, solver)
            
        if useDense:
            #p.debugmsg('dense SLE solver')
            try:
                C = p.C.toarray() if not isinstance(p.C, ndarray) else p.C
                if self.matrixSLEsolver == 'autoselect': 
                    self.matrixSLEsolver = linalg.solve
                xf = solver(C, p.d)
                istop, msg = 10, 'solved'
                p.xf = xf
                p.ff = norm(dot(C, xf)-p.d, inf)
            except linalg.LinAlgError:
                istop, msg = -10, 'singular matrix'
        else: # is sparse
            #p.debugmsg('sparse SLE solver')
            try:
                if not hasattr(p, 'C_as_csc'):p.C_as_csc = scipy.sparse.csc_matrix(p.C)
                xf = solver(p.C_as_csc, p.d)#, maxiter=10000)
                solver_istop = 0
                if solver_istop == 0:
                    istop, msg = 10, 'solved'
                else:
                    istop, msg = -104, 'the solver involved failed to solve the SLE'
                    if solver_istop < 0: msg += ', matter: illegal input or breakdown'
                    else: msg += ', matter: convergence to tolerance not achieved, number of iterations: %d' % solver_istop
                p.xf = xf                
                p.ff = norm(p.C_as_csc._mul_sparse_matrix(scipy.sparse.csr_matrix(xf.reshape(-1, 1))).toarray().flatten()-p.d, inf)                
            except:
                istop, msg = -100, 'unimplemented exception while solving sparse SLE'
        p.istop, p.msg = istop, msg




if __name__ == '__main__':
    import sys, os.path as pth
    sys.path.insert(0,pth.split(pth.split(pth.split(pth.split(pth.realpath(pth.dirname(__file__)))[0])[0])[0])[0])
    from openopt import NSP

    from numpy import ones, arange
    N = 4
    p = NSP(lambda x: abs(x).sum(), ones([N,1]), maxFunEvals = 1e6, plot = False)
    p.r0 = N
    r = p.solve('ShorEllipsoid')

from numpy import diag, ones, inf, any, copy, sqrt
from openopt.kernel.baseSolver import baseSolver
class ShorEllipsoid(baseSolver):
    __name__ = 'ShorEllipsoid'
    __license__ = "BSD"
    __authors__ = "Dmitrey"
    __alg__ = "Naum Z. Shor modificated method of ellipsoids"
    iterfcnConnected = True
    #__optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']

    def __init__(self): pass

    def __solver__(self, p):

        #if p.useScaling: p.err('ShorEllipsoid + scaling isn''t implemented yet')


        n = p.n

        B = diag(ones(n))

        x0 = copy(p.x0)

        ########################
        # Shor MME engine                         #
        # //by Dmitrey Kroshko                     #
        # //icq 275976670(inviz)                    #
        ########################

        x = x0.copy()
        xPrev = x0.copy()
        xf = x0.copy()
        xk = x0.copy()
        p.xk = x0.copy()

        if not hasattr(p, 'r0'):  p.err('ShorEllipsoid solver requires r0')
        r = p.r0

        f = p.f(x)

        fk = f
        ff = f
        p.fk = fk

        g = p.df(x)
        p._df = g

        p.iterfcn()
        if p.istop:
            p.xf = x
            p.ff = f
            #r.advanced.ralg.hs = hs
            return


        multiplier_R = sqrt(1.+1./n**2)
        beta = multiplier_R - 1. / n
        alfa = 1. / beta

        for k in range(p.maxIter):
            BTG = p.matmult(B.T, g.reshape(-1,1))
            dzeta_k = (BTG / p.norm(BTG))
            hk = r*beta/n
            x  -= hk * p.matmult(B, dzeta_k).flatten()
            B += p.matmult(B, p.matmult((1-alfa)*dzeta_k, dzeta_k.T))
            r *= multiplier_R
            f = p.f(x)
            g = p.df(x)
            xk = x.copy()
            fk = f


            if fk < ff:
                ff, xf = fk, xk.copy()

            p.fk = fk
            p.xk = xk
            p._df = g#CHECK ME! - MODIFIED!!
            p.iterfcn()
            if p.istop:
                p.xf = xf
                p.ff = ff
                #p.advanced.ralg.hs = max(norm(xPrev - x), hsmin)
                p._df = g
                return



from scipy.optimize import fmin_cobyla
#import openopt
from openopt.kernel.setDefaultIterFuncs import *
from openopt.kernel.ooMisc import xBounds2Matrix#,WholeRepr2LinConst
from openopt.kernel.baseSolver import baseSolver
from numpy import array, cumsum
#from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F

class EmptyClass: pass

class scipy_cobyla(baseSolver):
    __name__ = 'scipy_cobyla'
    __license__ = "BSD"
    __authors__ = """Michael JD Powell"""
    __alg__ = "Constrained Optimization BY Linear Approximation"
    __info__ = 'constrained NLP solver, no user-defined derivatives are handled'#TODO: add '__info__' field to other solvers
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    funcForIterFcnConnection = 'f'

    def __init__(self): pass
    def __solver__(self, p):
        #p.kernelIterFuncs.pop(SMALL_DELTA_X)
        #p.kernelIterFuncs.pop(SMALL_DELTA_F)
        xBounds2Matrix(p)

        det_arr = cumsum(array((p.nc, p.nh, p.b.size, p.beq.size, p.nh, p.beq.size)))

        cons = []
        for i in range(det_arr[-1]):
            if i < det_arr[0]:
                c = lambda x, i=i: - p.c(x)[i] # cobyla requires positive constraints!
            elif det_arr[0] <= i < det_arr[1]:
                j = i - det_arr[0]
                c = lambda x, j=j: p.h(x)[j]
            elif det_arr[1] <= i < det_arr[2]:
                j = i - det_arr[1]
                #assert 0<= j <p.cobyla.nb
                c = lambda x, j=j: p.b[j] - p.dotmult(p.A[j], x).sum() # cobyla requires positive constraints!
            elif det_arr[2] <= i < det_arr[3]:
                j = i - det_arr[2]
                #assert 0<= j <p.cobyla.nbeq
                c = lambda x, j=j: p.dotmult(p.Aeq[j], x).sum() - p.beq[j]
            elif det_arr[3] <= i < det_arr[4]:
                j = i - det_arr[3]
                c = lambda x, j=j: - p.h(x)[j]
            elif det_arr[4] <= i < det_arr[5]:
                j = i - det_arr[4]
                #assert 0<= j <p.cobyla.nbeq
                c = lambda x, j=j: p.dotmult(p.Aeq[j], x).sum() - p.beq[j]
            else:
                p.err('error in connection cobyla to openopt')
            cons.append(c)
##        def oo_cobyla_cons(x):
##            c0 = -p.c(x)
##            c1 = p.h(x)
##            c2 = -(p.matmult(p.A, x) - p.b)
##            c3 = p.matmult(p.Aeq, x) - p.beq
##            return hstack((c0, c1, -c1, c2, c3, -c3))


#        p.xk = p.x0.copy()
#        p.fk = p.f(p.x0)
#
#        p.iterfcn()
#        if p.istop:
#            p.xf = p.xk
#            p.ff = p.fk
#            return


        xf = fmin_cobyla(p.f, p.x0, cons = tuple(cons), iprint = 0, maxfun = p.maxFunEvals, rhoend = p.xtol )

        p.xk = xf
        p.fk = p.f(xf)
        p.istop = 1000
#        p.iterfcn()
#        p.xf = xf
#        p.ff = p.fk






"""
Solving system of equations:

x[0]**3 + x[1]**3 - 9 = 0
x[0] - 0.5*x[1] - 0.15*x[2]= 0
sinh(x[2]) + x[0] - 15 = 0
!! with numerical noise 1e-8 !!

Note that both fsolve and nssolve
get same gradient -
if no user-supplied one is available,
then same OO finite-difference one
is used (according to p.diffInt value)

If you have matplotlib installed,
you'll get a figure.
Typical fsolve fails number
(for scipy 0.6.0)
is ~ 10-15%

This test runs ~ a minute on my AMD 3800+
"""
noise = 1e-8

from openopt import SNLE
from numpy import asfarray, zeros, cos, sin, arange, cosh, sinh, log10, ceil, floor, arange, inf, nanmin, nanmax
from time import time
from scipy import rand

x0 = [8, 15, 0.80]
global count1, count2, count3
count1 = count2 = count3 = 0
def Count1():
    global count1; count1 +=1; return 0
def Count2():
    global count2; count2 +=1; return 0
def Count3():
    global count3; count3 +=1; return 0

# ATTENTION!
# when no user-supplied gradient is available
# nssolve can take benefites from splitting funcs
# so using f=[fun1, fun2, fun3, ...](or f=(fun1, fun2, fun3, ...))
# (where each fun is separate equation)
# is more appriciated than
# f = lambda x: (...)
# or def f(x): (...)

f_without_noise = \
[lambda x: x[0]**2+x[1]**2-9, lambda x: x[0]-0.5*x[1] - 0.15*x[2], lambda x: sinh(x[2])+x[0]-15]
def fvn(x):
    r = -inf
    for f in f_without_noise: r = max(r, abs(f(x)))
    return r
f = [lambda x: x[0]**2+x[1]**2-9 + noise*rand(1)+Count1(), lambda x: x[0]-0.5*x[1] - 0.15*x[2] + noise*rand(1)+Count2(), \
lambda x: sinh(x[2])+x[0]-15 + noise*rand(1)+Count3()]# + (2007 * x[3:]**2).tolist()

#optional: gradient
##def DF(x):
##    df = zeros((3,3))
##    df[0,0] = 3*x[0]**2
##    df[0,1] = 3*x[1]**2
##    df[1,0] = 1
##    df[1,1] = -0.5
##    df[1,2] = -0.15
##    df[2,0] = 1
##    df[2,2] = cosh(x[2])
##    return df

N = 100
desired_ftol = 1e-6
assert desired_ftol - noise*len(x0) > 1e-7
#w/o gradient:
scipy_fsolve_failed, fs = 0, []
print '----------------------------------'
print 'desired ftol:', desired_ftol, 'objFunc noise:', noise
############################################################################
print '---------- fsolve fails ----------'
t = time()
print 'N log10(MaxResidual) MaxResidual'
for i in xrange(N):
    p = SNLE(f, x0, ftol = desired_ftol - noise*len(x0), iprint = -1, maxFunEvals = int(1e7))
    r = p.solve('scipy_fsolve')
    v = fvn(r.xf)
    fs.append(log10(v))
    if v > desired_ftol:
        scipy_fsolve_failed += 1
        print i+1, '       %0.2f       ' % log10(v), v
    else:
        print i+1, 'OK'
print 'fsolve time elapsed', time()-t
#print 'fsolve_failed number:', scipy_fsolve_failed , '(from', N, '),', 100.0*scipy_fsolve_failed / N, '%'
print 'counters:', count1, count2, count3
############################################################################
count1 = count2 = count3 = 0
t = time()
print '---------- nssolve fails ---------'
nssolve_failed, ns = 0, []
print 'N log10(MaxResidual) MaxResidual'
for i in xrange(N):
    p = SNLE(f, x0, ftol = desired_ftol - noise*len(x0), iprint = -1, maxFunEvals = int(1e7))
    r = p.solve('nssolve')
    #r = p.solve('nlp:amsg2p')
    #r = p.solve('nlp:ralg')
    v = fvn(r.xf)
    ns.append(log10(v))
    if  v > desired_ftol:
        nssolve_failed += 1
        print i+1, '       %0.2f       ' % log10(v), v
    else:
        print i+1, 'OK'
print 'nssolve time elapsed', time()-t
print 'nssolve_failed number:', nssolve_failed , '(from', N, '),', 100.0 * nssolve_failed / N, '%'
print 'counters:', count1, count2, count3
############################################################################
print '------------ SUMMARY -------------'
print 'fsolve_failed number:', scipy_fsolve_failed , '(from', N, '),', 100.0*scipy_fsolve_failed / N, '%'
print 'nssolve_failed number:', nssolve_failed , '(from', N, '),', 100.0 * nssolve_failed / N, '%'

#try:
from pylab import *
subplot(2,1,1)
grid(1)
title('scipy.optimize fsolve fails to achive desired ftol: %0.1f%%' %(100.0*scipy_fsolve_failed / N))
xmin1, xmax1 = floor(nanmin(fs)), ceil(nanmax(fs))+1
hist(fs, arange(xmin1, xmax1))
#xlabel('log10(maxResidual)')
axvline(log10(desired_ftol), color='green', linewidth=3, ls='--')
[ymin1, ymax1] = ylim()

################
subplot(2,1,2)
grid(1)
title('openopt nssolve fails to achive desired ftol: %0.1f%%' % (100.0*nssolve_failed / N))
xmin2, xmax2 = floor(nanmin(ns)), ceil(nanmax(ns))+1
#hist(ns, 5)
hist(ns, arange(xmin2, xmax2))
xlabel('log10(maxResidual)')
axvline(log10(desired_ftol), color='green', linewidth=3, ls='--')
[ymin2, ymax2] = ylim()

################
xmin, xmax = min(xmin1, xmin2) - 0.1, max(xmax1, xmax2) + 0.1
ymin, ymax = 0, max(ymax1, ymax2) * 1.05
subplot(2,1,1)
xlim(xmin, xmax)
ylim(0, ymax)
subplot(2,1,2)
xlim(xmin, xmax)
show()
#except:
#    pass

